@@ -1,5 +1,5 @@
 # GNU Solfege - eartraining for GNOME
-# Copyright (C) 2000, 2001, 2002  Tom Cato Amundsen
+# Copyright (C) 2000, 2001, 2002, 2003  Tom Cato Amundsen
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -16,6 +16,7 @@
 # Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 
 """
+FIXME update.
 In addition to the names in this file, only this is public in
 the mpd module:
 
@@ -31,8 +32,10 @@ from const import FIRST, LAST, ALL
 import parser, soundcard
 import string, types
 from musicalpitch import MusicalPitch
-from intervall import Intervall
-from utils import notename_to_int, int_to_notename
+from interval import Interval
+from mpdutils import notename_to_int, int_to_notename
+
+from _exceptions import MpdException
 
 def music_to_tracklist(music, velocity):
     """
