@@ -1,5 +1,5 @@
 # GNU Solfege - eartraining for GNOME
-# Copyright (C) 2000, 2001, 2002  Tom Cato Amundsen
+# Copyright (C) 2000, 2001, 2002, 2003  Tom Cato Amundsen
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -18,7 +18,7 @@
 from soundcard.rat import Rat
 import const
 import gtk
-import utils
+import mpdutils
 import operator
 
 fetadir = "feta"
@@ -50,8 +50,6 @@ class Engraver:
     def __init__(self, timepos, fontsize):
         self.m_fontsize = 20
         self.m_timepos = timepos
-    def get_pixmap_id(self):
-        raise "Abstract base class"
     def get_width(self):
         # this value will be used for engravers that don't override this method.
         # FIXME
@@ -68,7 +66,7 @@ class ClefEngraver(Engraver):
         self.m_xpos = dict[self.m_timepos].m_clef
     def get_width(self):
         return 25
-    def engrave(self, w, pixmap, gc, staff_yoffset):
+    def engrave(self, widget, gc, staff_yoffset):
         fn = {'violin': 'G',
               'treble': 'G',
               'subbass': 'F',
@@ -80,12 +78,13 @@ class ClefEngraver(Engraver):
               'mezzosoprano': 'C',
               'soprano': 'C',
               'french': 'G'}[self.m_cleftype]
-        xpm = gtk.create_pixmap_from_xpm(w, None,
-              fetadir+'/feta%i-clefs-%s.xpm' % (self.m_fontsize, fn))[0]
-        gtk.draw_pixmap(pixmap, gc, xpm, 0, 0,
+        im = gtk.Image()
+        im.set_from_file(fetadir+'/feta%i-clefs-%s.xpm' % (self.m_fontsize, fn))
+        im.get_pixbuf().render_to_drawable(widget.window, gc, 0, 0,
                     self.m_xpos,
                     dimentions[self.m_fontsize].clef_yoffset[self.m_cleftype] \
-                     + staff_yoffset, -1, -1)
+                     + staff_yoffset, -1, -1,
+                     gtk.gdk.RGB_DITHER_NONE, 0, 0)
 
 
 class TimeSignatureEngraver(Engraver):
@@ -94,22 +93,24 @@ class TimeSignatureEngraver(Engraver):
         self.m_timesig = timesig
     def set_xpos(self, dict):
         self.m_xpos = dict[self.m_timepos].m_timesignature
-    def engrave(self, w, pixmap, gc, staff_yoffset):
+    def engrave(self, widget, gc, staff_yoffset):
         x = 0
         for idx in range(len(str(self.m_timesig.m_num))):
             n = str(self.m_timesig.m_num)[idx]
-            xpm = gtk.create_pixmap_from_xpm(w, None,
-                fetadir+'/feta%i-number-%s.xpm' % (self.m_fontsize, n))[0]
-            gtk.draw_pixmap(pixmap, gc, xpm, 0, 0,
-                    self.m_xpos+x, staff_yoffset-12, -1, -1)
+            im = gtk.Image()
+            im.set_from_file(fetadir+'/feta%i-number-%s.xpm' % (self.m_fontsize, n))
+            im.get_pixbuf().render_to_drawable(widget.window, gc, 0, 0,
+                    self.m_xpos+x, staff_yoffset-12, -1, -1,
+                    gtk.gdk.RGB_DITHER_NONE, 0, 0)
             x  += 10
         x = 0
         for idx in range(len(str(self.m_timesig.m_den))):
             n = str(self.m_timesig.m_den)[idx]
-            xpm = gtk.create_pixmap_from_xpm(w, None,
-                fetadir+'/feta%i-number-%s.xpm' % (self.m_fontsize, n))[0]
-            gtk.draw_pixmap(pixmap, gc, xpm, 0, 0,
-                    self.m_xpos+x, staff_yoffset+3, -1, -1)
+            im = gtk.Image()
+            im.set_from_file(fetadir+'/feta%i-number-%s.xpm' % (self.m_fontsize, n))
+            im.get_pixbuf().render_to_drawable(widget.window, gc, 0, 0,
+                    self.m_xpos+x, staff_yoffset+3, -1, -1,
+                    gtk.gdk.RGB_DITHER_NONE, 0, 0)
             x += 10
     def __str__(self):
         return '(TimeSignatureEngraver:%i/%i, xpos:%s)' % (self.m_timesig.m_den,
@@ -130,15 +131,15 @@ class TieEngraver(Engraver):
         #FIXME the musicdisplayer want every engraver to have a m_xpos to
         # know how wide the whole music is. Fixthis
         self.m_xpos = self.m_xpos2
-    def engrave(self, w, pixmap, gc, staff_yoffset):
+    def engrave(self, widget, gc, staff_yoffset):
         dim = dimentions[self.m_fontsize]
         self.m_xpos1 += 3 + (self.m_shift1 + 1) * dim.xshift
         self.m_xpos2 += - 3 + self.m_shift2 * dim.xshift
         w = self.m_xpos2 - self.m_xpos1
         h = 8
-        gtk.draw_arc(pixmap, gc, gtk.FALSE,
-                     self.m_xpos1, staff_yoffset + dim.linespacing*(self.m_ylinepos-1)/2 + 3,
-                     w, h, 64 * 180, 64*180)
+        widget.window.draw_arc(gc, gtk.FALSE,
+              self.m_xpos1, staff_yoffset + dim.linespacing*(self.m_ylinepos-1)/2 + 3,
+              w, h, 64 * 180, 64*180)
 
 
 class AccidentalsEngraver(Engraver):
@@ -155,7 +156,7 @@ class AccidentalsEngraver(Engraver):
         self.m_xpos = dict[self.m_timepos].m_accidentals + tpd[self.m_timepos].m_accidentals - self.get_width()
     def get_width(self):
         return self.m_usize
-    def engrave(self, w, pixmap, gc, staff_yoffset):
+    def engrave(self, widget, gc, staff_yoffset):
         x = 0
         for y in self.m_accs.keys():
             for acc in self.m_accs[y]:
@@ -164,16 +165,15 @@ class AccidentalsEngraver(Engraver):
                             const.ACCIDENTAL_0: -5,
                             const.ACCIDENTAL_1: -6,
                             const.ACCIDENTAL_2: -2}
-                xpm = gtk.create_pixmap_from_xpm(w, None,
-                    fetadir+'/feta%i-accidentals-%i.xpm' % (self.m_fontsize,
-                              acc))[0]
-                gtk.draw_pixmap(pixmap, gc,
-                        xpm, 0, 0,
+                im = gtk.Image()
+                im.set_from_file(fetadir+'/feta%i-accidentals-%i.xpm' % (self.m_fontsize, acc))
+                im.get_pixbuf().render_to_drawable(widget.window, gc,
+                        0, 0,
                         self.m_xpos + x,
                         y_offset[acc] + staff_yoffset
                          + dimentions[self.m_fontsize].linespacing*y/2
                          + y_offset[acc],
-                        -1, -1)
+                        -1, -1, gtk.gdk.RGB_DITHER_NONE, 0, 0)
                 x += dimentions[self.m_fontsize].accidental_widths[acc] + 1
 
 
@@ -182,17 +182,17 @@ class KeySignatureEngraver(Engraver):
         Engraver.__init__(self, timepos, fontsize)
         self.m_key = key
         self.m_clef = clef
-        self.m_accidentals = utils.key_to_accidentals(key)
+        self.m_accidentals = mpdutils.key_to_accidentals(key)
     def set_xpos(self, dict):
         self.m_xpos = dict[self.m_timepos].m_keysignature
     def get_width(self):
         #FIXME this value depends on the fontsize, and on what kind of
         #accidental we are drawing. A sharp is wider that a flat.
         return len(self.m_accidentals) * 10 + 10
-    def engrave(self, w, pixmap, gc, staff_yoffset):
+    def engrave(self, widget, gc, staff_yoffset):
         x = 0
         for acc in self.m_accidentals:
-            ypos = utils.an_to_ylinepos(acc, self.m_clef)
+            ypos = mpdutils.an_to_ylinepos(acc, self.m_clef)
             if acc[-4:] == 'eses':
                 type = -2
             elif acc[-2:] == 'es':
@@ -207,15 +207,14 @@ class KeySignatureEngraver(Engraver):
                     const.ACCIDENTAL_0: -5,
                     const.ACCIDENTAL_1: -6,
                     const.ACCIDENTAL_2: -2}
-            xpm = gtk.create_pixmap_from_xpm(w, None,
-                fetadir+'/feta%i-accidentals-%i.xpm' % (self.m_fontsize, type))[0]
-            gtk.draw_pixmap(pixmap, gc,
-                    xpm, 0, 0,
+            im = gtk.Image()
+            im.set_from_file(fetadir+'/feta%i-accidentals-%i.xpm' % (self.m_fontsize, type))
+            im.get_pixbuf().render_to_drawable(widget.window, gc, 0, 0,
                     self.m_xpos + x,
                     y_offset[type] + staff_yoffset
                      + dimentions[self.m_fontsize].linespacing*ypos/2
-                     + y_offset[type],
-                    -1, -1)
+                     + y_offset[type], -1, -1,
+                     gtk.gdk.RGB_DITHER_NONE, 0, 0)
             #FIXME see FIXME msg in .get_width
             x += 10
 
@@ -238,22 +237,22 @@ class NoteheadEngraver(Engraver):
         self.m_mgroup = mgroup
     def set_xpos(self, dict):
         self.m_xpos = dict[self.m_timepos].m_music
-    def engrave(self, w, pixmap, gc, staff_yoffset):
+    def engrave(self, widget, gc, staff_yoffset):
         dim = dimentions[self.m_fontsize]
-        xpm = gtk.create_pixmap_from_xpm(w, None,
-                fetadir+'/feta%i-noteheads-%i.xpm' % (self.m_fontsize, self.m_head))[0]
-        gtk.draw_pixmap(pixmap, gc,
-                    xpm, 0, 0,
+        im = gtk.Image()
+        im.set_from_file(fetadir+'/feta%i-noteheads-%i.xpm' % (self.m_fontsize, self.m_head))
+        im.get_pixbuf().render_to_drawable(widget.window, gc, 0, 0,
                     self.m_xpos + self.m_shift * dim.xshift,
                     staff_yoffset + dim.linespacing*self.m_ypos/2 - 4,
-                    -1, -1)
-        dot = gtk.create_pixmap_from_xpm(w, None, fetadir+'/feta20-dots-dot.xpm')[0]
+                    -1, -1,
+                     gtk.gdk.RGB_DITHER_NONE, 0, 0)
+        im.set_from_file(fetadir+'/feta20-dots-dot.xpm')
         for n in range(self.m_numdots):
-            gtk.draw_pixmap(pixmap, gc,
-                        dot, 0, 0,
+            im.get_pixbuf().render_to_drawable(widget.window, gc, 0, 0,
                         self.m_xpos+dim.xshift*(self.m_shift+1.5+n/2.0),
                         -3 + staff_yoffset + dim.linespacing*self.m_ypos/2,
-                        -1, -1)
+                        -1, -1,
+                     gtk.gdk.RGB_DITHER_NONE, 0, 0)
     def get_pixmap_id(self):
         return {0: const.NOTEHEAD_0,
                 1: const.NOTEHEAD_1,
@@ -262,6 +261,7 @@ class NoteheadEngraver(Engraver):
         return "(NoteheadEngraver: xpos:%i, ypos:%i, head:%i)" % (self.m_xpos,
                                                                     self.m_ypos,
                                                                     self.m_head)
+
 class BarlineEngraver(Engraver):
     def __init__(self, timepos, fontsize, type):
         Engraver.__init__(self, timepos, fontsize)
@@ -270,11 +270,10 @@ class BarlineEngraver(Engraver):
         self.m_xpos = dict[self.m_timepos].m_barline
     def get_width(self):
         return 8
-    def engrave(self, w, pixmap, gc, staff_yoffset):
+    def engrave(self, widget, gc, staff_yoffset):
         dim = dimentions[self.m_fontsize]
-        gtk.draw_line(pixmap, gc,
-                self.m_xpos, staff_yoffset - dim.linespacing*2,
-                self.m_xpos, staff_yoffset + dim.linespacing*2)
+        widget.window.draw_line(gc, self.m_xpos, staff_yoffset - dim.linespacing*2,
+                                    self.m_xpos, staff_yoffset + dim.linespacing*2)
 
 class TupletEngraver(Engraver):
     def __init__(self, fontsize, n):
@@ -300,7 +299,7 @@ class TupletEngraver(Engraver):
         self.m_b = max(bottom) + 2
         self.m_xpos1 = self.m_stems[0].m_xpos
         self.m_xpos2 = self.m_stems[-1].m_xpos + dim.xshift
-    def engrave(self, w, pixmap, gc, staff_yoffset):
+    def engrave(self, widget, gc, staff_yoffset):
         dim = dimentions[self.m_fontsize]
         m = self.m_xpos1 + (self.m_xpos2 - self.m_xpos1)/2
         x1 = m - 6
@@ -313,12 +312,13 @@ class TupletEngraver(Engraver):
             y = max(staff_yoffset + dim.linespacing * 5,
                     staff_yoffset + self.m_b * dim.linespacing /2)
             d = -1
-        gtk.draw_line(pixmap, gc, self.m_xpos1, y, x1, y)
-        gtk.draw_line(pixmap, gc, x2, y, self.m_xpos2, y)
-        gtk.draw_line(pixmap, gc, self.m_xpos1, y, self.m_xpos1, y+5*d)
-        gtk.draw_line(pixmap, gc, self.m_xpos2, y, self.m_xpos2, y+5*d)
+        widget.window.draw_line(gc, self.m_xpos1, y, x1, y)
+        widget.window.draw_line(gc, x2, y, self.m_xpos2, y)
+        widget.window.draw_line(gc, self.m_xpos1, y, self.m_xpos1, y+5*d)
+        widget.window.draw_line(gc, self.m_xpos2, y, self.m_xpos2, y+5*d)
+        #UGH shouldn't we use pango font string to be more portable?
         font = gtk.load_font("-adobe-helvetica-medium-r-normal-*-12-*-*-*-*-*-*-*")
-        gtk.draw_string(pixmap, font, gc, m-3, y + 3, str(self.m_den))
+        widget.window.draw_string(font, gc, m-3, y + 3, str(self.m_den))
 
 class BeamEngraver(Engraver):
     def __init__(self, fontsize):
@@ -370,8 +370,9 @@ class BeamEngraver(Engraver):
         self.m_stemdir = stemdir
         for e in self.m_stems:
             e.m_mgroup.m_stemdir = stemdir
-    def engrave(self, w, pixmap, gc, staff_yoffset):
+    def engrave(self, widget, gc, staff_yoffset):
         dim = dimentions[self.m_fontsize]
+        d = 0
         if self.m_stemdir == const.UP:
            d = 1
         else:
@@ -381,26 +382,24 @@ class BeamEngraver(Engraver):
         if self.m_stems[0].m_beamed_stem_top % 2 == 1:
             for x in range(len(self.m_stems)):
                 self.m_stems[x].m_beamed_stem_top =\
-                  + self.m_stems[x].m_beamed_stem_top - d
+                    self.m_stems[x].m_beamed_stem_top - d
         y1 = self.m_stems[0].m_beamed_stem_top * dim.linespacing/2 + staff_yoffset
         beamw = 3
         beaml = 10
         for y in range(beamw):
-            gtk.draw_line(pixmap, gc, x1, y1 + y*d, x2, y1 + y*d)
+            widget.window.draw_line(gc, x1, y1 + y*d, x2, y1 + y*d)
         for stem in self.m_stems:
             stem._beam_done = 8
-        def short_beam(stem, xdir, beamnum, gc=gc, pixmap=pixmap,
-                       beamw=beamw, beaml=beaml, y1=y1, d=d):
+        def short_beam(stem, xdir, beamnum, d=d, widget=widget, beamw=beamw, gc=gc, beaml=beaml, y1=y1):
             for y in range(beamw):
-                gtk.draw_line(pixmap, gc,
-                       stem.m_xpos, y1 + y + beamnum*d*beamw*2,
+                widget.window.draw_line(gc, stem.m_xpos, y1 + y + beamnum*d*beamw*2,
                        stem.m_xpos + xdir*beaml, y1 + y + beamnum*d*beamw*2)
         for nl, yc in ((16, 0), (32, 1), (64, 2)):
             for i in range(len(self.m_stems)-1):
                 if self.m_stems[i].m_mgroup.m_duration.m_nh >= nl \
                         and self.m_stems[i+1].m_mgroup.m_duration.m_nh >= nl:
                     for y in range(beamw):
-                        gtk.draw_line(pixmap, gc,
+                        widget.window.draw_line(gc,
                             self.m_stems[i].m_xpos, d*beamw*yc*2 + y1 + y + d*beamw*2,
                             self.m_stems[i+1].m_xpos, d*beamw*yc*2 + y1 + y + d*beamw*2)
                     self.m_stems[i]._beam_done = nl
@@ -416,11 +415,11 @@ class BeamEngraver(Engraver):
                             dir = 1
                         else:
                             dir = -1
-                    short_beam(self.m_stems[i], dir, yc+1)
+                    short_beam(self.m_stems[i], dir, yc+1, d)
                     self.m_stems[i]._beam_done = nl
             if self.m_stems[-1].m_mgroup.m_duration.m_nh >= nl \
                  and self.m_stems[-1]._beam_done <= nl/2:
-                 short_beam(self.m_stems[-1], -1, yc+1)
+                 short_beam(self.m_stems[-1], -1, yc+1, d)
 
 
 class StemEngraver(Engraver):
@@ -447,26 +446,24 @@ class StemEngraver(Engraver):
     def calc_xpos(self):
         if self.m_mgroup.m_stempos == 1 or self.m_mgroup.m_stemdir == const.UP:
             self.m_xpos = self.m_xpos + dimentions[self.m_fontsize].xshift
-    def engrave(self, w, pixmap, gc, staff_yoffset):
+    def engrave(self, widget, gc, staff_yoffset):
         dim = dimentions[self.m_fontsize]
         # draw flags
-        if  not self.m_is_beamed and self.m_mgroup.m_duration.m_nh > 4:
+        if not self.m_is_beamed and self.m_mgroup.m_duration.m_nh > 4:
             if self.m_mgroup.m_stemdir == const.UP:
-                xpm = gtk.create_pixmap_from_xpm(w, None,
-                    fetadir+'/feta%i-flags-%s.xpm' % (self.m_fontsize, self.get_flag(const.UP)))[0]
-                gtk.draw_pixmap(pixmap, gc,
-                            xpm, 0, 0,
+                im = gtk.Image()
+                im.set_from_file(fetadir+'/feta%i-flags-%s.xpm' % (self.m_fontsize, self.get_flag(const.UP)))
+                im.get_pixbuf().render_to_drawable(widget.window, widget.get_style().black_gc, 0, 0,
                             self.m_xpos,
                             staff_yoffset - self.m_stemlen + self.m_yposes[0] * dim.linespacing/2,
-                            -1, -1)
+                            -1, -1, gtk.gdk.RGB_DITHER_NONE, 0, 0)
             else:
-                xpm = gtk.create_pixmap_from_xpm(w, None,
-                    fetadir+'/feta%i-flags-%s.xpm' % (self.m_fontsize, self.get_flag(const.DOWN)))[0]
-                gtk.draw_pixmap(pixmap, gc,
-                            xpm, 0, 0,
+                im = gtk.Image()
+                im.set_from_file(fetadir+'/feta%i-flags-%s.xpm' % (self.m_fontsize, self.get_flag(const.DOWN)))
+                im.get_pixbuf().render_to_drawable(widget.window, widget.get_style().black_gc, 0, 0,
                             self.m_xpos,
                             staff_yoffset + 4 + self.m_yposes[-1] * dim.linespacing/2,
-                            -1, -1)
+                            -1, -1, gtk.gdk.RGB_DITHER_NONE, 0, 0)
         # draw stem
         if self.m_mgroup.m_stemdir == const.DOWN:
             if self.m_is_beamed:
@@ -480,7 +477,7 @@ class StemEngraver(Engraver):
             else:
                 yroot = self.m_yposes[0] - 6
             ytop = self.m_yposes[-1]
-        gtk.draw_line(pixmap, gc,
+        widget.window.draw_line(gc,
                      self.m_xpos,
                      ytop * dim.linespacing/2 + staff_yoffset,
                      self.m_xpos,
@@ -511,16 +508,16 @@ class LedgerLineEngraver(Engraver):
         self.m_down = down
     def set_xpos(self, dict):
         self.m_xpos = dict[self.m_timepos].m_music
-    def engrave(self, w, pixmap, gc, staff_yoffset):
+    def engrave(self, widget, gc, staff_yoffset):
         dim = dimentions[self.m_fontsize]
         if self.m_up:
             for y in range(self.m_up):
-                gtk.draw_line(pixmap, gc,
+                widget.window.draw_line(gc,
                     self.m_xpos+dim.ledger_left, (-y-3) * dim.linespacing + staff_yoffset,
                     self.m_xpos+dim.ledger_right, (-y-3) * dim.linespacing + staff_yoffset)
         if self.m_down:
             for y in range(self.m_down):
-                gtk.draw_line(pixmap, gc,
+                widget.window.draw_line(gc,
                     self.m_xpos+dim.ledger_left, (y+3) * dim.linespacing + staff_yoffset,
                     self.m_xpos+dim.ledger_right, (y+3) * dim.linespacing + staff_yoffset)
     def __str__(self):
@@ -541,19 +538,20 @@ class RestEngraver(Engraver):
     def get_width(self):
         #FIXME write me!
         return 20
-    def engrave(self, w, pixmap, gc, staff_yoffset):
+    def engrave(self, widget, gc, staff_yoffset):
         dim = dimentions[self.m_fontsize]
-        xpm = gtk.create_pixmap_from_xpm(w, None, 
-                fetadir+'/feta%i-rests-%i.xpm' % (self.m_fontsize, self.m_type))[0]
+        im = gtk.Image()
+        im.set_from_file(fetadir+'/feta%i-rests-%i.xpm' % (self.m_fontsize, self.m_type))
         if self.m_type == 0:
             my = dim.linespacing/2
         else:
             my = 0
-        gtk.draw_pixmap(pixmap, gc,
-                    xpm, 0, 0,
-                    self.m_xpos, 
+        im.get_pixbuf().render_to_drawable(widget.window, widget.get_style().black_gc,
+                    0, 0,
+                    self.m_xpos,
                     staff_yoffset - my + dim.linespacing*self.m_ypos/2 - 4,
-                    -1, -1)
+                    -1, -1,
+                    gtk.gdk.RGB_DITHER_NONE, 0, 0)
     def get_pixmap_id(self):
         return {1: const.REST_1,
                 2: const.REST_2,
