@@ -1,5 +1,5 @@
 # GNU Solfege - eartraining for GNOME
-# Copyright (C) 2000, 2001, 2002  Tom Cato Amundsen
+# Copyright (C) 2000, 2001, 2002, 2003  Tom Cato Amundsen
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -44,7 +44,15 @@ from soundcard.rat import Rat
 import const
 import soundcard
 import types
-import utils
+import mpdutils
+import _exceptions
+
+class ParseError(_exceptions.MpdException):
+    def __init__(self, toc):
+        self.m_toc = toc
+    def __str__(self):
+        return "Parse error: %s" % self.m_toc
+
 
 def musicalpitch_relative(first, second):
     """
@@ -57,9 +65,9 @@ def musicalpitch_relative(first, second):
     """
     assert isinstance(first, MusicalPitch)
     assert isinstance(second, MusicalPitch)
-    n1 = MusicalPitch(second)
+    n1 = second.clone()
     n1.m_octave_i = first.m_octave_i
-    n2 = MusicalPitch(n1)
+    n2 = n1.clone()
     if n1 < first:
         n2.m_octave_i += 1
     else:
@@ -158,7 +166,7 @@ class Score:
                     # stems
                     v = []
                     for n in voice.m_coldict[timepos].m_music.values():
-                        v.append(utils.steps_to_ylinepos(n.m_pitch.steps(), clef))
+                        v.append(mpdutils.steps_to_ylinepos(n.m_pitch.steps(), clef))
                     if v and (voice.m_coldict[timepos].m_music.values()[0].m_duration.m_nh > 1):
                         v.sort()
                         if voice.m_coldict[timepos].m_beaminfo == 'start':
@@ -232,7 +240,7 @@ class Score:
                         m = voice.m_coldict[timepos].m_music[musicalpitch].m_pitch
                         e = staff.needed_accidental(m)
                         if e is not None:
-                            v[utils.steps_to_ylinepos(m.steps(), clef)] = e
+                            v[mpdutils.steps_to_ylinepos(m.steps(), clef)] = e
                 if v:
                     se.append(AccidentalsEngraver(timepos, fontsize, v))
                     self.m_timeposdict[timepos].m_accidentals \
@@ -270,7 +278,7 @@ class Score:
                     #   the notehead.
                     nd = {}
                     for musicalpitch in voice.m_coldict[timepos].m_music.keys():
-                        nd[utils.steps_to_ylinepos(voice.m_coldict[timepos].m_music[musicalpitch].m_pitch.steps(), clef)] = voice.m_coldict[timepos].m_music[musicalpitch]
+                        nd[mpdutils.steps_to_ylinepos(voice.m_coldict[timepos].m_music[musicalpitch].m_pitch.steps(), clef)] = voice.m_coldict[timepos].m_music[musicalpitch]
                     if nd == {}:
                         # nd == {} when there are not noteheads, for example
                         # when there is a rest here.
@@ -314,7 +322,7 @@ class Score:
                             head = const.NOTEHEAD_2
                         else:
                             head = const.NOTEHEAD_1
-                        ylinepos = utils.steps_to_ylinepos(m.m_pitch.steps(), clef)
+                        ylinepos = mpdutils.steps_to_ylinepos(m.m_pitch.steps(), clef)
                         se.append(NoteheadEngraver(timepos, fontsize, m.m_shift,
                              ylinepos, head, m.m_duration.m_dots,
                              m.m_pitch.semitone_pitch(),
@@ -340,7 +348,7 @@ class Score:
                         continue
                     for k in voice.m_coldict[timepos].m_music.keys():
                         ypos = voice.m_coldict[timepos].m_music[k].m_pitch.steps()
-                        ypos = utils.steps_to_ylinepos(ypos, clef)
+                        ypos = mpdutils.steps_to_ylinepos(ypos, clef)
                         if up > ypos < -5:
                             up = ypos
                         if down < ypos > 5:
@@ -364,7 +372,7 @@ class Score:
                     sh2 = tie[4].m_shift
                     se.append(TieEngraver(fontsize, tie[0], tie[1],
                                           sh1, sh2,
-                                          utils.steps_to_ylinepos(tie[2][0], clef)))
+                                          mpdutils.steps_to_ylinepos(tie[2][0], clef)))
         #########################################
         xv = {}
         p = 0
@@ -408,8 +416,6 @@ class Score:
         staff = Staff(self)
         self.m_staffs.append(staff)
         return staff
-    def display(self):
-        print self.m_timeposdict
     def get_midi_events(self, velocity, start=None, end=None):
         kv = self.m_timeposdict.keys()
         kv.sort()
@@ -456,7 +462,7 @@ class Score:
         for staff in self.m_staffs:
             for voice in staff.m_voice_list:
                track_list.append(voice.generate_track_for_voice(staff, kv, chan, velocity))
-               #chan = chan + 1
+               chan = chan + 1
         return track_list
 
 
@@ -475,8 +481,8 @@ class Staff:
         self.m_i = {}
         for step in range(MusicalPitch.LOWEST_STEPS, MusicalPitch.HIGHEST_STEPS+1):
             self.m_i[step] = 0
-        for a in utils.key_to_accidentals(key):
-            n = MusicalPitch(a)
+        for a in mpdutils.key_to_accidentals(key):
+            n = MusicalPitch.new_from_notename(a)
             for oct in range(-4, 7):
                 n.m_octave_i = oct
                 if n.semitone_pitch() < 128:
@@ -691,10 +697,10 @@ def parse_to_score_object(music):
                 cur_staff.announce_timepos(timepos)
                 cur_staff.add_timesignature(toc_data, timepos)
             elif toc == Lexer.KEY:
-                p = MusicalPitch(toc_data[0])
+                p = MusicalPitch.new_from_notename(toc_data[0])
                 if transpose_pitch:
                     p.transpose_by_musicalpitch(transpose_pitch)
-                k = (p.notename(), toc_data[1])
+                k = (p.get_notename(), toc_data[1])
                 cur_staff.announce_timepos(timepos)
                 cur_staff.add_keysignature(timepos, k)
             elif toc == Lexer.TIMES:
@@ -703,7 +709,7 @@ def parse_to_score_object(music):
                     cur_voice.announce_timepos(timepos)
                     cur_voice.start_tuplet(timepos, times, tuplet_dir)
                 else:
-                    raise r"\times nn/nn does not nest"
+                    raise ParseError(r"\times nn/nn does not nest")
             elif toc == Lexer.CLEF:
                 cur_staff.announce_timepos(timepos)
                 cur_staff.add_clef(timepos, toc_data)
@@ -715,17 +721,17 @@ def parse_to_score_object(music):
                     context = NOTES
                     if not cur_staff.m_coldict[Rat(0, 1)].m_keysignature:
                         if transpose_pitch:
-                            k = (transpose_pitch.notename(), 'major')
+                            k = (transpose_pitch.get_notename(), 'major')
                         else:
                             k = ('c', 'major')
                         cur_staff.add_keysignature(Rat(0, 1), k)
                 else:
-                    raise "parse error {"
+                    raise ParseError("Token '{' not allowed here.")
             elif toc == '<':
                 if context == NOTES:
                     context = START_OF_CHORD
                 else:
-                    raise "'<' not allowed here"
+                    raise ParseError("Token '<' not allowed here.")
             elif toc == '>':
                 if context == CHORD:
                     if tie_is_in_the_air:
@@ -737,7 +743,7 @@ def parse_to_score_object(music):
                     relto = relto_backup; relto_backup = None
                     context = NOTES
                 else:
-                    raise "error parsing '>'"
+                    raise ParseError("Token '>' not allowed here.")
             elif toc == '}':
                 if context == NOTES:
                     if times:
@@ -746,7 +752,7 @@ def parse_to_score_object(music):
                     else:
                         context = TOPLEVEL
                 else:
-                    raise "error parsing '}'"
+                    raise ParseError("Token '}' not allowed here.")
             elif toc == '[':
                 cur_voice.announce_timepos(timepos)
                 cur_voice.start_beam(timepos)
@@ -765,7 +771,7 @@ def parse_to_score_object(music):
                 if relative_mode:
                     toc_data.m_pitch = musicalpitch_relative(
                                               relto, toc_data.m_pitch)
-                    relto = MusicalPitch(toc_data.m_pitch)
+                    relto = toc_data.m_pitch.clone()
                 if transpose_pitch:
                     toc_data.transpose(transpose_pitch)
                 cur_voice.add_notehead(timepos, toc_data, stem_dir)
@@ -794,7 +800,7 @@ def parse_to_score_object(music):
             else:
                 print "*%s*" % toc, type(toc), lexer.m_idx
                 print "music:", music[lexer.m_idx-4:]
-                raise "Parse error", toc
+                raise ParseError(toc)
         return score
 
 
@@ -870,7 +876,8 @@ class Lexer:
             else:
                 skiplen = 0
                 if dots:
-                    raise "Error"
+                    #FIXME
+                    raise ParseError("Error")
             numdots = len(dots)
             if skiplen is 0:
                 retval = self.m_notelen
@@ -888,7 +895,7 @@ class Lexer:
                 notelen = 0
                 if dots:
                     #FIXME
-                    raise "LexerError"
+                    raise ParseError("LexerError")
             n = Music(notename, notelen, numdots)
             if not n.m_duration:
                 n.m_duration = self.m_notelen.clone()
@@ -906,7 +913,7 @@ class Lexer:
         m = self.re_relative.match(self.m_string, self.m_idx)
         if m:
             self.m_idx += len(m.group())
-            return self.RELATIVE, MusicalPitch(m.group(1))
+            return self.RELATIVE, MusicalPitch.new_from_notename(m.group(1))
         m = self.re_clef.match(self.m_string, self.m_idx)
         if m:
             self.m_idx += len(m.group())
@@ -924,7 +931,7 @@ class Lexer:
         m = self.re_transpose.match(self.m_string, self.m_idx)
         if m:
             self.m_idx += len(m.group())
-            return self.TRANSPOSE, MusicalPitch(m.group(1))
+            return self.TRANSPOSE, MusicalPitch.new_from_notename(m.group(1))
         m = self.re_time.match(self.m_string, self.m_idx)
         if m:
             self.m_idx += len(m.group())
