@@ -108,3 +108,12 @@ class MidiFileSynth:
         v = v + mfutils.mf_end_of_track()
         mfutils.write_vect(f, v)
         f.close()
+    def stop(self):
+        if sys.platform != 'win32':
+            if self.__child_pid:
+                try:
+                    os.kill(self.__child_pid, signal.SIGKILL)
+                except OSError:
+                    pass
+                os.wait()
+                self.__child_pid = None
