@@ -22,7 +22,8 @@ def gcd(a, b):
         a, b = b, a % b
     return a
 
-class Rat:
+class Rat(object):
+    __slots__ = ('m_num', 'm_den')
     """
     This is NOT a generic rational number class, it includes
     only the features needed for the the soundcard and mpd
