@@ -1,5 +1,5 @@
 # GNU Solfege - eartraining for GNOME
-# Copyright (C) 2000, 2001, 2002  Tom Cato Amundsen
+# Copyright (C) 2000, 2001, 2002, 2003  Tom Cato Amundsen
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -17,8 +17,6 @@
 
 import __main__
 
-from i18n import _
-
 import configureoutput
 import errno, sys
 import shutil
@@ -36,6 +34,7 @@ class SolfegeApp(cfg.ConfigUtils):
         self.check_rcfile()
         self.check_statistics()
         self.m_teachers = {}
+        self.m_running_exercise = None
         self.lessonfile_manager = lessonfile.LessonFileManager()
         self.setup_sound()
     def check_rcfile(self):
@@ -73,7 +72,8 @@ class SolfegeApp(cfg.ConfigUtils):
                 cfg.set_string("id-tone/new_ak", "<ctrl>n")
             if cfg.get_string("id-tone/give_up_ak") == "":
                 cfg.set_string("id-tone/give_up_ak", "<ctrl>g")
-            cfg.set_bool("gui/development_menu_visible", 0)
+            if cfg.get_int("melodic-interval/maximum_number_of_intervals") < 1:
+                cfg.set_string("melodic-interval/maximum_number_of_intervals", 10)
         cfg.set_string('app/version', configureoutput.VERSION_STRING)
     def check_statistics(self):
         # the devel series are fresh and no history to care about
@@ -162,16 +162,30 @@ code to handle situations like this.)"""
     def please_help_me(self):
         if self.m_ui.m_viewer != 'docviewer':
             self.handle_href('%s-help.html' % self.m_ui.m_viewer)
-    def handle_href(self, href):
-        protocol, action, fn, lessoncollection, lessonfile, config = \
+    def handle_href(self, href, display_docfile_set_adj=1):
+        if self.m_ui.m_viewer:
+            if display_docfile_set_adj == 1:
+                # we get here when we click on urls, because we want to
+                # save the place in the current document when we load the
+                # new one. We don't get here if handle_href is called from
+                # history_back.
+                self.m_ui.m_history.set_adj_of_current(self.m_ui.box_dict['docviewer'].get_vadjustment().value)
+        protocol, action, fn, anchor, lessoncollection, lessonfile, config = \
            utils.parse_url(href)
         if protocol == None:
-            self.m_ui.display_docfile(fn)
+            self.m_ui.display_docfile(fn, anchor)
         elif protocol == 'solfege':
             if action == 'all-lessonfiles':
                 self.m_ui.display_html(self.lessonfile_manager.m_htmldoc)
                 self.m_ui.m_history.add('solfege:all-lessonfiles')
             else:
+                if (self.m_ui.m_viewer == 'docviewer'
+                    and self.m_running_exercise != None
+                    and self.m_running_exercise != fn) or \
+                   (self.m_ui.m_viewer != 'docviewer'):
+                       self.m_ui.box_dict[self.m_running_exercise].on_end_practise()
+                       self.m_running_exercise = None
+                _old_viewer = self.m_ui.m_viewer
                 if not self.m_teachers.has_key(fn):
                     n = utils.exercise_name_to_module_name(fn)
                     exec("import %s" % n)
@@ -182,6 +196,11 @@ code to handle situations like this.)"""
                     self.m_teachers[fn].g_view = self.m_ui.box_dict[fn]
                 self.m_teachers[fn].configure_exercise(lessoncollection, lessonfile, config)
                 self.m_ui.activate_exercise(href)
+                if not (_old_viewer == 'docviewer'
+                        and self.m_running_exercise != None
+                        and self.m_running_exercise == fn):
+                    self.m_ui.box_dict[self.m_ui.m_viewer].on_start_practise()
+                self.m_running_exercise = fn
         elif protocol == 'http':
             os.system(self.get_string("config/web_browser") % href)
         elif protocol == 'mailto':
