@@ -1,5 +1,5 @@
 # GNU Solfege - eartraining for GNOME
-# Copyright (C) 2000, 2001, 2002  Tom Cato Amundsen
+# Copyright (C) 2000, 2001, 2002, 2003  Tom Cato Amundsen
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -20,7 +20,7 @@ TODO:
     - cfg.set_list
     - watches outside ConfigUtils
 >>> import cfg
->>> cfg.initialise("default.config", "~/.cfg-test")
+>>> cfg.initialise(None, "default.config", "~/.cfg-test")
 >>> cfg.get_int("gui/button_height")
 40
 >>> cfg.set_string("section/stringvar", "value")
@@ -38,6 +38,9 @@ TODO:
 >>> cfg.set_bool("section/boolvar", 0)
 >>> cfg.get_bool("section/boolvar")
 0
+>>> cfg.set_list("section/listv", [1, 2, 3])
+>>> cfg.get_list("section/listv")
+[1, 2, 3]
 >>> c = ConfigUtils("section")
 >>> c.get_string("section/stringvar")
 'value'
@@ -47,7 +50,7 @@ TODO:
 0
 >>> c.get_float("section/floatvar") == 3.141592
 1
->>> cfg.initialise(None, "~/.solfege-cfg-testing")
+>>> cfg.initialise(None, None, "~/.solfege-cfg-testing")
 >>> c.get_string("ss/a")
 ''
 >>> c.get_string("ss/a=hei")
@@ -59,6 +62,7 @@ TODO:
 >>> c.get_list("ss/a=[1, 2, 3]")
 [1, 2, 3]
 """
+
 import string, re
 import types
 import os, os.path, sys
@@ -100,7 +104,8 @@ def parse_file_into_dict(dict, filename):
         elif comment_m:
             pass
         else:
-           raise "X"
+           print filename
+           raise "X", line
         line = file.readline()
     file.close()
     return dict
@@ -123,30 +128,45 @@ def set_string(key, val):
     global data
     _maybe_create_key(key)
     section, k = split(key)
+    oldval = get_string(key)
     data[section][k] = val
+    if _watches.has_key(key) and (oldval != get_string(key)):
+        for cb in _watches[key].values():
+            cb(key)
 
 def set_int(key, val):
-    global data
+    global data, _watches
     assert type(val) == types.IntType
     _maybe_create_key(key)
     section, k = split(key)
+    oldval = get_string(key)
     data[section][k] = str(val)
+    if _watches.has_key(key) and (oldval != get_string(key)):
+        for cb in _watches[key].values():
+            cb(key)
 
 def set_float(key, val):
-    global data
+    global data, _watches
     assert type(val) == types.FloatType
     _maybe_create_key(key)
     section, k = split(key)
+    oldval = get_string(key)
     data[section][k] = str(val)
+    if _watches.has_key(key) and (oldval != get_string(key)):
+        for cb in _watches[key].values():
+            cb(key)
 
 def set_bool(key, val):
-    global data
+    global data, _watches
     _maybe_create_key(key)
     if val:
         set_string(key, "true")
     else:
         set_string(key, "false")
 
+def set_list(key, val):
+    global data, _watches
+    set_string(key, str(val))
 
 ########################
 ## get_XXX functions ##
@@ -170,6 +190,9 @@ def get_int(key):
       default = 0
    section, k = split(key)
    try:
+       # UGH win32 fix
+       if not data[section][k]:
+           return 0
        return int(float(data[section][k]))
    except KeyError:
        return int(default)
@@ -195,6 +218,9 @@ def get_bool(key):
         print "warning: cfg.get_bool: value of '%s' is not true or false" % key
         return 0
 
+def get_list(key):
+    return eval(get_string(key))
+
 ######################################
 ## reading and writing the database ##
 ######################################
@@ -252,7 +278,7 @@ def initialise(app_defaults_filename, system_filename, user_filename):
         data = parse_file_into_dict(data, user_filename)
 
 
-class ConfigUtils:
+class ConfigUtils(object):
     def __init__(self, exname):
         self.m_exname = exname
     def add_watch(self, name, callback):
@@ -261,7 +287,7 @@ class ConfigUtils:
         if not _watches.has_key(name):
             _watches[name] = {}
         _watches[name][_watch_counter] = callback
-        _watch_counter = _watch_counter + 1
+        _watch_counter += 1
         return _watch_counter - 1
     def remove_watch(self, name, id):
         global _watches, _watch_counter
@@ -279,27 +305,30 @@ class ConfigUtils:
         if string.count(name, "/") == 0:
             return "%s/%s" % (self.m_exname, name)
         return name
-    def _set(self, func, name, val):
-        global _watches
-        name = self._expand_name(name)
-        oldval = get_string(name)
-        func(name, val)
-        newval = get_string(name)
-        if _watches.has_key(name) and (oldval != newval):
-            for cb in _watches[name].values():
-               cb(name)
-    def set_int(self, name, val):
-        self._set(set_int, name, val)
-    def set_float(self, name, val):
-        self._set(set_float, name, val)
+    ######
+    # set
+    ######
     def set_string(self, name, val):
-        self._set(set_string, name, val)
-    def set_list(self, name, val):
-        self._set(set_string, name, str(val))
+        set_string(self._expand_name(name), val)
+    def set_int(self, name, val):
+        set_int(self._expand_name(name), val)
+    def set_float(self, name, val):
+        set_float(self._expand_name(name), val)
     def set_bool(self, name, val):
-        self._set(set_bool, name, val)
+        set_bool(self._expand_name(name), val)
+    def set_list(self, name, val):
+        set_list(self._expand_name(name), val)
+    ######
+    # get
+    ######
     def _get(self, func, name, default=""):
         return func(self._expand_name(name)+default)
+    def get_string(self, name):
+        r = self._get(get_string, name)
+        if r is None:
+            return ""
+        else:
+            return r
     def get_int(self, name):
         return self._get(get_int, name)
     def get_int_with_default(self, name, default):
@@ -307,18 +336,12 @@ class ConfigUtils:
         return self.get_int(name+"=%i" % default)
     def get_float(self, name):
         return self._get(get_float, name)
-    def get_string(self, name):
-        r = self._get(get_string, name)
-        if r is None:
-            return ""
-        else:
-            return r
+    def get_bool(self, name):
+        return self._get(get_bool, name)
     def get_list(self, name):
         if '=' in name:
            return eval(self._get(get_string, name))
         return eval(self._get(get_string, name, "=[]"))
-    def get_bool(self, name):
-        return self._get(get_bool, name)
 
 
 if __name__ == '__main__':
