@@ -1,5 +1,5 @@
 # GNU Solfege - eartraining for GNOME
-# Copyright (C) 2000, 2001, 2002  Tom Cato Amundsen
+# Copyright (C) 2000, 2001, 2002, 2003  Tom Cato Amundsen
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -21,16 +21,15 @@ import gu, mpd
 import configureoutput
 import widgets
 from instrumentselector import ReadOnlyInstrumentSelector
-from i18n import _
 import cfg
 import soundcard
 
-class ConfigWindow(gnome.ui.GnomePropertyBox, cfg.ConfigUtils):
+class ConfigWindow(gnome.ui.PropertyBox, cfg.ConfigUtils):
     def on_destroy(self, widget):
         self.m_app.m_ui.g_config_window.destroy()
         self.m_app.m_ui.g_config_window = None
     def __init__(self, app):
-        gnome.ui.GnomePropertyBox.__init__(self)
+        gnome.ui.PropertyBox.__init__(self)
         cfg.ConfigUtils.__init__(self, 'configwindow')
         self.m_app = app
         self._is_alive = 0
@@ -40,20 +39,20 @@ class ConfigWindow(gnome.ui.GnomePropertyBox, cfg.ConfigUtils):
         ########
         # midi #
         ########
-        vbox = gtk.GtkVBox()
-        self.append_page(vbox, gtk.GtkLabel(_("Midi stuff")))
+        vbox = gtk.VBox()
+        self.append_page(vbox, gtk.Label(_("Midi stuff")))
         hbox = gu.bHBox(vbox, gtk.FALSE)
-        hbox.set_border_width(gnome.uiconsts.PAD)
+        hbox.set_border_width(gnome.ui.PAD)
 
-        table = gtk.GtkTable(1, 1, gtk.FALSE)
+        table = gtk.Table(1, 1, gtk.FALSE)
         hbox.pack_start(table, gtk.FALSE)
 
         gu.tLabel(table, 0, 1, 0, 1, _("Slow bpm:"), 1.0,
-                  xoptions=gtk.FILL, xpadding=gnome.uiconsts.PAD)
+                  xoptions=gtk.FILL, xpadding=gnome.ui.PAD)
         gu.tLabel(table, 0, 1, 1, 2, _("Default bpm:"), 1.0, 0.5,
-                  xoptions=gtk.FILL, xpadding=gnome.uiconsts.PAD)
+                  xoptions=gtk.FILL, xpadding=gnome.ui.PAD)
         gu.tLabel(table, 0, 1, 2, 3, _("Arpeggio bpm:"), 1.0, 0.5,
-                  xoptions=gtk.FILL, xpadding=gnome.uiconsts.PAD)
+                  xoptions=gtk.FILL, xpadding=gnome.ui.PAD)
         self.g_slow_bpm = gu.tSpinButton(table, 1, 2, 0, 1,
                                          self.get_int('config/slow_bpm'),
                                          10, 500, 1, 10,
@@ -67,40 +66,40 @@ class ConfigWindow(gnome.ui.GnomePropertyBox, cfg.ConfigUtils):
                                              10, 500, 1, 10, self.on_changed)
         self.g_instrsel = ReadOnlyInstrumentSelector(_("Preferred instrument"),
                'config', 'preferred_instrument', self.on_changed)
-        self.g_instrsel.set_border_width(gnome.uiconsts.PAD)
+        self.g_instrsel.set_border_width(gnome.ui.PAD)
         hbox.pack_start(self.g_instrsel, gtk.FALSE)
         ########
         # user #
         ########
-        box = gtk.GtkVBox()
-        box.set_border_width(gnome.uiconsts.PAD)
-        self.append_page(box, gtk.GtkLabel(_("User")))
+        box = gtk.VBox()
+        box.set_border_width(gnome.ui.PAD)
+        self.append_page(box, gtk.Label(_("User")))
         hbox = gu.bHBox(box, gtk.FALSE)
 
-        table = gtk.GtkTable(2, 3, gtk.FALSE)
+        table = gtk.Table(2, 3, gtk.FALSE)
         hbox.pack_start(table, gtk.FALSE)
         gu.tLabel(table, 0, 1, 0, 1, _("Highest note user can sing:"), 1.0,
-                  xoptions=gtk.FILL, xpadding=gnome.uiconsts.PAD_SMALL)
+                  xoptions=gtk.FILL, xpadding=gnome.ui.PAD_SMALL)
         gu.tLabel(table, 0, 1, 1, 2, _("Lowest note user can sing:"), 1.0,
-                  xoptions=gtk.FILL, xpadding=gnome.uiconsts.PAD_SMALL)
+                  xoptions=gtk.FILL, xpadding=gnome.ui.PAD_SMALL)
 
-        self.g_highest_singable = widgets.SimplestNotenameSpinButton(
+        self.g_highest_singable = widgets.NotenameSpinButton(
             self.get_string('user/highest_pitch'))
-        self.g_highest_singable.g_entry.connect('changed', self.on_changed)
+        self.g_highest_singable.connect('value-changed', self.on_changed)
         table.attach(self.g_highest_singable, 1, 2 , 0, 1)
-        self.g_lowest_singable = widgets.SimplestNotenameSpinButton(
+        self.g_lowest_singable = widgets.NotenameSpinButton(
             self.get_string('user/lowest_pitch'))
-        self.g_lowest_singable.g_entry.connect('changed', self.on_changed)
+        self.g_lowest_singable.connect('value-changed', self.on_changed)
         table.attach(self.g_lowest_singable, 1, 2, 1, 2)
         widgets.NotenameRangeController(
                   self.g_lowest_singable, self.g_highest_singable,
                   mpd.LOWEST_NOTENAME, mpd.HIGHEST_NOTENAME)
 
         hbox = gu.bHBox(box, gtk.FALSE)
-        self.g_sex_male = gtk.GtkRadioButton(None, _("Male"))
+        self.g_sex_male = gtk.RadioButton(None, _("Male"))
         self.g_sex_male.connect('clicked', self.on_changed)
         hbox.pack_start(self.g_sex_male, gtk.FALSE)
-        self.g_sex_female = gtk.GtkRadioButton(self.g_sex_male, _("Female"))
+        self.g_sex_female = gtk.RadioButton(self.g_sex_male, _("Female"))
         self.g_sex_female.connect('clicked', self.on_changed)
         hbox.pack_start(self.g_sex_female, gtk.FALSE)
         if self.get_string('user/sex') == 'female':
@@ -108,94 +107,87 @@ class ConfigWindow(gnome.ui.GnomePropertyBox, cfg.ConfigUtils):
         #######
         # gui #
         #######
-        vbox = gtk.GtkVBox()
-        vbox.set_border_width(gnome.uiconsts.PAD)
-        self.append_page(vbox, gtk.GtkLabel(_("Gui")))
+        vbox = gtk.VBox()
+        vbox.set_border_width(gnome.ui.PAD)
+        self.append_page(vbox, gtk.Label(_("Gui")))
 
         hbox = gu.bHBox(vbox, gtk.FALSE)
-        hbox.set_spacing(gnome.uiconsts.PAD)
+        hbox.set_spacing(gnome.ui.PAD)
 
-        frame = gtk.GtkFrame(_("Main toolbar"))
-        frame.set_border_width(gnome.uiconsts.PAD)
+        frame = gtk.Frame(_("Main toolbar"))
         hbox.pack_start(frame, gtk.FALSE)
-        frame_vbox = gtk.GtkVBox()
-        frame_vbox.set_border_width(gnome.uiconsts.PAD)
-        frame.add(frame_vbox)
-        frame_hbox = gu.bHBox(frame_vbox)
+        table = gtk.Table()
+        table.set_border_width(gnome.ui.PAD)
+        frame.add(table)
 
         self.g_toolbar_icons = gu.RadioButton(None, _("Icons"), self.on_changed)
-        frame_hbox.pack_start(self.g_toolbar_icons, gtk.FALSE)
+        table.attach(self.g_toolbar_icons, 0, 1, 0, 1)
         self.g_toolbar_text = gu.RadioButton(self.g_toolbar_icons, _("Text"), self.on_changed)
-        frame_hbox.pack_start(self.g_toolbar_text, gtk.FALSE)
+        table.attach(self.g_toolbar_text, 1, 2, 0, 1)
         self.g_toolbar_both = gu.RadioButton(self.g_toolbar_icons, _("Both"), self.on_changed)
-        frame_hbox.pack_start(self.g_toolbar_both, gtk.FALSE)
-        sep = gtk.GtkHSeparator()
-        frame_vbox.pack_start(sep)
+        table.attach(self.g_toolbar_both, 2, 3, 0, 1)
+        sep = gtk.HSeparator()
+        table.attach(sep, 0, 3, 1, 2)
         self.__dict__['g_toolbar_'+self.get_string('gui/toolbar_style=icons')].set_active(gtk.TRUE)
-        frame_hbox = gu.bHBox(frame_vbox, gtk.FALSE)
         self.g_toolbar_show = gu.RadioButton(None, _("Show"), self.on_changed)
-        frame_hbox.pack_start(self.g_toolbar_show, gtk.FALSE)
+        table.attach(self.g_toolbar_show, 0, 1, 2, 3)
         self.g_toolbar_hide = gu.RadioButton(self.g_toolbar_show, _("Hide"), self.on_changed)
-        frame_hbox.pack_start(self.g_toolbar_hide, gtk.FALSE)
+        table.attach(self.g_toolbar_hide, 1, 2, 2, 3)
         if self.get_bool('gui/toolbar_visible'):
             self.g_toolbar_show.set_active(gtk.TRUE)
         else:
             self.g_toolbar_hide.set_active(gtk.TRUE)
+        self.add_watch('gui/toolbar_visible', self._watch_toolbar_radios)
 
-        frame = gtk.GtkFrame(_("Navigation toolbar"))
-        frame.set_border_width(gnome.uiconsts.PAD)
+        frame = gtk.Frame(_("Navigation toolbar"))
         hbox.pack_start(frame, gtk.FALSE)
-        frame_vbox = gtk.GtkVBox()
-        frame_vbox.set_border_width(gnome.uiconsts.PAD)
-        frame.add(frame_vbox)
-        frame_hbox = gu.bHBox(frame_vbox)
-        sep = gtk.GtkHSeparator()
-        frame_vbox.pack_start(sep)
+        table = gtk.Table()
+        table.set_border_width(gnome.ui.PAD)
+        frame.add(table)
 
         self.g_navbar_icons = gu.RadioButton(None, _("Icons"), self.on_changed)
-        frame_hbox.pack_start(self.g_navbar_icons, gtk.FALSE)
+        table.attach(self.g_navbar_icons, 0, 1, 0, 1)
         self.g_navbar_text = gu.RadioButton(self.g_navbar_icons, _("Text"), self.on_changed)
-        frame_hbox.pack_start(self.g_navbar_text, gtk.FALSE)
+        table.attach(self.g_navbar_text, 1, 2, 0, 1)
         self.g_navbar_both = gu.RadioButton(self.g_navbar_icons, _("Both"), self.on_changed)
-        frame_hbox.pack_start(self.g_navbar_both, gtk.FALSE)
+        table.attach(self.g_navbar_both, 2, 3, 0, 1)
+        sep = gtk.HSeparator()
+        table.attach(sep, 0, 3, 1, 2)
         self.__dict__['g_navbar_'+self.get_string('gui/navbar_style=icons')].set_active(gtk.TRUE)
-        frame_hbox = gu.bHBox(frame_vbox, gtk.FALSE)
         self.g_navbar_show = gu.RadioButton(None, _("Show"), self.on_changed)
-        frame_hbox.pack_start(self.g_navbar_show, gtk.FALSE)
+        table.attach(self.g_navbar_show, 0, 1, 2, 3)
         self.g_navbar_hide = gu.RadioButton(self.g_navbar_show, _("Hide"), self.on_changed)
-        frame_hbox.pack_start(self.g_navbar_hide, gtk.FALSE)
+        table.attach(self.g_navbar_hide, 1, 2, 2, 3)
         if self.get_bool('gui/navbar_visible'):
             self.g_navbar_show.set_active(gtk.TRUE)
         else:
             self.g_navbar_hide.set_active(gtk.TRUE)
+        self.add_watch('gui/navbar_visible', self._watch_navbar_radios)
 
-        self.g_mainwin_user_resizeable = gtk.GtkCheckButton(
+        self.g_mainwin_user_resizeable = gtk.CheckButton(
               _("User resizeable main window"))
         self.g_mainwin_user_resizeable.connect('clicked', self.on_changed)
         self.g_mainwin_user_resizeable.set_active(
                         self.get_bool('gui/mainwin_user_resizeable'))
+        self.add_watch('gui/mainwin_user_resizeable',
+                lambda key: self.g_mainwin_user_resizeable.set_active(
+                            self.get_bool('gui/mainwin_user_resizeable')))
         vbox.pack_start(self.g_mainwin_user_resizeable, gtk.FALSE)
 
-        self.g_development_menu_visible = gtk.GtkCheckButton(
-            _("Show development menu"))
-        self.g_development_menu_visible.connect('clicked', self.on_changed)
-        self.g_development_menu_visible.set_active(
-                       self.get_bool('gui/development_menu_visible'))
-        vbox.pack_start(self.g_development_menu_visible, gtk.FALSE)
         self._is_alive = 1
 
-        hbox = gu.bHBox(vbox)
-        hbox.set_spacing(gnome.uiconsts.PAD_SMALL)
-        hbox.pack_start(gtk.GtkLabel(_("Web browser:")), gtk.FALSE)
-        self.g_web_browser = gtk.GtkEntry()
+        hbox = gu.bHBox(vbox, gtk.FALSE)
+        hbox.set_spacing(gnome.ui.PAD_SMALL)
+        hbox.pack_start(gtk.Label(_("Web browser:")), gtk.FALSE)
+        self.g_web_browser = gtk.Entry()
         self.g_web_browser.set_text(self.get_string("config/web_browser"))
         self.g_web_browser.connect('changed', self.on_changed)
         hbox.pack_start(self.g_web_browser)
 
-        hbox = gu.bHBox(vbox)
-        hbox.set_spacing(gnome.uiconsts.PAD_SMALL)
-        hbox.pack_start(gtk.GtkLabel(_("Mail program:")), gtk.FALSE)
-        self.g_mail_program = gtk.GtkEntry()
+        hbox = gu.bHBox(vbox, gtk.FALSE)
+        hbox.set_spacing(gnome.ui.PAD_SMALL)
+        hbox.pack_start(gtk.Label(_("Mail program:")), gtk.FALSE)
+        self.g_mail_program = gtk.Entry()
         self.g_mail_program.set_text(self.get_string("config/mua"))
         self.g_mail_program.connect('changed', self.on_changed)
         hbox.pack_start(self.g_mail_program)
@@ -203,15 +195,15 @@ class ConfigWindow(gnome.ui.GnomePropertyBox, cfg.ConfigUtils):
         ############
         # Practise #
         ############
-        vbox = gtk.GtkVBox()
-        vbox.set_border_width(gnome.uiconsts.PAD)
-        self.append_page(vbox, gtk.GtkLabel(_("Practise")))
+        vbox = gtk.VBox()
+        vbox.set_border_width(gnome.ui.PAD)
+        self.append_page(vbox, gtk.Label(_("Practise")))
 
-        self.g_picky_on_new_question = gtk.GtkCheckButton(_("Not allow new question before the old is solved"))
+        self.g_picky_on_new_question = gtk.CheckButton(_("Not allow new question before the old is solved"))
         self.g_picky_on_new_question.set_active(self.get_bool('config/picky_on_new_question'))
         vbox.pack_start(self.g_picky_on_new_question, gtk.FALSE)
 
-        self.g_autorepeat_if_wrong = gtk.GtkCheckButton(_("Repeat question if the answer was wrong"))
+        self.g_autorepeat_if_wrong = gtk.CheckButton(_("Repeat question if the answer was wrong"))
         self.g_autorepeat_if_wrong.set_active(self.get_bool('config/auto_repeat_question_if_wrong_answer'))
         vbox.pack_start(self.g_autorepeat_if_wrong, gtk.FALSE)
 
@@ -226,11 +218,21 @@ class ConfigWindow(gnome.ui.GnomePropertyBox, cfg.ConfigUtils):
         else:
             self.create_linux_sound_page()
         self.show_all()
+    def _watch_toolbar_radios(self, key):
+        if self.get_bool('gui/toolbar_visible'):
+            self.g_toolbar_show.set_active(gtk.TRUE)
+        else:
+            self.g_toolbar_hide.set_active(gtk.TRUE)
+    def _watch_navbar_radios(self, key):
+        if self.get_bool('gui/navbar_visible'):
+            self.g_navbar_show.set_active(gtk.TRUE)
+        else:
+            self.g_navbar_hide.set_active(gtk.TRUE)
     def create_linux_sound_page(self):
-        vbox = gtk.GtkVBox()
-        vbox.set_border_width(gnome.uiconsts.PAD)
-        self.append_page(vbox, gtk.GtkLabel(_("Sound setup")))
-        txt = gtk.GtkLabel(_("""Solfege has two ways to play music. It can use /dev/music or an external midi player program. The preferred way is to use /dev/music since this offers features that will be utilized more in later versions of this program."""))
+        vbox = gtk.VBox()
+        vbox.set_border_width(gnome.ui.PAD)
+        self.append_page(vbox, gtk.Label(_("Sound setup")))
+        txt = gtk.Label(_("""Solfege has two ways to play music. It can use /dev/music or an external midi player program. The preferred way is to use /dev/music since this offers features that will be utilized more in later versions of this program."""))
         txt.set_line_wrap(1)
         txt.set_justify(gtk.JUSTIFY_FILL)
         txt.set_alignment(0.0, 0.0)
@@ -241,12 +243,12 @@ class ConfigWindow(gnome.ui.GnomePropertyBox, cfg.ConfigUtils):
         self.g_device_radio = gu.RadioButton(self.g_fakesynth_radio,
               _("Use device"), None)
         hbox.pack_start(self.g_device_radio, gtk.FALSE)
-        combo = gtk.GtkCombo()
+        combo = gtk.Combo()
         combo.set_popdown_strings(('/dev/sequencer', '/dev/sequencer2',
                                    '/dev/music'))
         self.g_device_entry = combo.entry
         self.g_device_entry.set_text(self.get_string('sound/device_file'))
-        self.g_synth_num = gtk.GtkSpinButton(gtk.GtkAdjustment(0, 0, 100, 1, 1),
+        self.g_synth_num = gtk.SpinButton(gtk.Adjustment(0, 0, 100, 1, 1),
                              digits=0)
         self.g_synth_num.set_value(self.get_int('sound/synth_number'))
         hbox.pack_start(combo, gtk.FALSE)
@@ -256,7 +258,7 @@ class ConfigWindow(gnome.ui.GnomePropertyBox, cfg.ConfigUtils):
         self.g_midiplayer_radio = gu.RadioButton(self.g_fakesynth_radio,
              _("Use external midiplayer"), None)
         hbox.pack_start(self.g_midiplayer_radio, gtk.FALSE)
-        combo = gtk.GtkCombo()
+        combo = gtk.Combo()
         combo.set_popdown_strings(('drvmidi -i d %s',
                                    'timidity -idqq %s',
                                    'playmidi -f %s'))
@@ -271,26 +273,26 @@ class ConfigWindow(gnome.ui.GnomePropertyBox, cfg.ConfigUtils):
             self.g_fakesynth_radio.set_active(gtk.TRUE)
 
         # checkbox to enable awe support
-        self.g_awe_checkbutton = gtk.GtkCheckButton(_("My sound card is Sound Blaster AWE32, AWE64 or pnp32"))
+        self.g_awe_checkbutton = gtk.CheckButton(_("My sound card is Sound Blaster AWE32, AWE64 or pnp32"))
         self.g_awe_checkbutton.set_active(self.get_string('sound/card_info') == 'awe')
         if not configureoutput.HAVE_LINUX_AWE_VOICE_H:
             self.g_awe_checkbutton.set_sensitive(gtk.FALSE)
-        vbox.pack_start(self.g_awe_checkbutton)
+        vbox.pack_start(self.g_awe_checkbutton, gtk.FALSE)
 
-        gu.bButton(vbox, _("Sound test. Hit 'Apply' before testing."), self.on_sound_test)
+        gu.bButton(vbox, _("Sound test. Hit 'Apply' before testing."), self.on_sound_test, gtk.FALSE, gtk.FALSE)
 
         self.g_fakesynth_radio.connect('clicked', self.on_changed)
         self.g_midiplayer_radio.connect('clicked', self.on_changed)
         self.g_device_radio.connect('clicked', self.on_changed)
         self.g_midiplayer_entry.connect('changed', self.on_changed)
         self.g_device_entry.connect('changed', self.on_changed)
-        self.g_synth_num.connect('changed', self.on_changed)
+        self.g_synth_num.connect('value-changed', self.on_changed)
         self.g_awe_checkbutton.connect('clicked', self.on_changed)
     def create_win32_sound_page(self):
-        vbox = gtk.GtkVBox()
-        vbox.set_border_width(gnome.uiconsts.PAD)
-        self.append_page(vbox, gtk.GtkLabel(_("Sound setup")))
-        txt = gtk.GtkLabel(_("""Solfege has two ways to play music. It is recommended to use Windows multimedia output. Support for external midiplayer is only available because this was the only way to get sound on Windows in earlier versions of this program."""))
+        vbox = gtk.VBox()
+        vbox.set_border_width(gnome.ui.PAD)
+        self.append_page(vbox, gtk.Label(_("Sound setup")))
+        txt = gtk.Label(_("""Solfege has two ways to play music. It is recommended to use Windows multimedia output. An external midiplayer is only useful if your soundcard lack a hardware synth, and you have to use a program like timidity to play the music."""))
         txt.set_line_wrap(1)
         txt.set_justify(gtk.JUSTIFY_FILL)
         txt.set_alignment(0.0, 0.0)
@@ -300,7 +302,7 @@ class ConfigWindow(gnome.ui.GnomePropertyBox, cfg.ConfigUtils):
         hbox = gu.bHBox(vbox, gtk.FALSE)
         self.g_device_radio = gu.RadioButton(self.g_fakesynth_radio,
               _("Windows multimedia output, synth number:"), None)
-        self.g_synth_num = gtk.GtkSpinButton(gtk.GtkAdjustment(0, 0, 100, 1, 1),
+        self.g_synth_num = gtk.SpinButton(gtk.Adjustment(0, 0, 100, 1, 1),
                              digits=0)
         self.g_synth_num.set_value(self.get_int('sound/synth_number'))
         hbox.pack_start(self.g_device_radio, gtk.FALSE)
@@ -310,7 +312,7 @@ class ConfigWindow(gnome.ui.GnomePropertyBox, cfg.ConfigUtils):
         self.g_midiplayer_radio = gu.RadioButton(self.g_fakesynth_radio,
              _("Use external midiplayer"), None)
         hbox.pack_start(self.g_midiplayer_radio, gtk.FALSE)
-        combo = gtk.GtkCombo()
+        combo = gtk.Combo()
         combo.set_popdown_strings(('midiplayer.exe %s',
                                    'timidity -idqq %s',
                                    'playmidi -f %s'))
@@ -360,73 +362,64 @@ class ConfigWindow(gnome.ui.GnomePropertyBox, cfg.ConfigUtils):
                          self.g_instrsel.g_velocity.get_value_as_int())
         if pagenum == 1:
             self.set_string('user/lowest_pitch',
-                            self.g_lowest_singable.get_value())
+                mpd.int_to_notename(self.g_lowest_singable.get_value_as_int()))
             self.set_string('user/highest_pitch',
-                            self.g_highest_singable.get_value())
-            if self.g_sex_male.active:
+                mpd.int_to_notename(self.g_highest_singable.get_value_as_int()))
+            if self.g_sex_male.get_active():
                 self.set_string('user/sex', 'male')
             else:
                 self.set_string('user/sex', 'female')
         if pagenum == 2:
-            if self.g_toolbar_text.active:
+            if self.g_toolbar_text.get_active():
                 self.m_app.m_ui.g_toolbar.set_style(gtk.TOOLBAR_TEXT)
                 self.set_string('gui/toolbar_style', 'text')
-            elif self.g_toolbar_icons.active:
+            elif self.g_toolbar_icons.get_active():
                 self.m_app.m_ui.g_toolbar.set_style(gtk.TOOLBAR_ICONS)
                 self.set_string('gui/toolbar_style', 'icons')
-            elif self.g_toolbar_both.active:
+            elif self.g_toolbar_both.get_active():
                 self.m_app.m_ui.g_toolbar.set_style(gtk.TOOLBAR_BOTH)
                 self.set_string('gui/toolbar_style', 'both')
-            if self.g_toolbar_show.active:
-                self.m_app.m_ui.get_dock_item_by_name('main-toolbar').show()
+            if self.g_toolbar_show.get_active():
+                self.m_app.m_ui.get_dock_item_by_name('toolbar').show()
                 self.set_bool('gui/toolbar_visible', 1)
             else:
                 self.set_bool('gui/toolbar_visible', 0)
-                self.m_app.m_ui.get_dock_item_by_name('main-toolbar').hide()
+                self.m_app.m_ui.get_dock_item_by_name('toolbar').hide()
 
-            if self.g_navbar_text.active:
+            if self.g_navbar_text.get_active():
                 self.m_app.m_ui.g_navbar.set_style(gtk.TOOLBAR_TEXT)
                 self.set_string('gui/navbar_style', 'text')
-            elif self.g_navbar_icons.active:
+            elif self.g_navbar_icons.get_active():
                 self.m_app.m_ui.g_navbar.set_style(gtk.TOOLBAR_ICONS)
                 self.set_string('gui/navbar_style', 'icons')
-            elif self.g_navbar_both.active:
+            elif self.g_navbar_both.get_active():
                 self.m_app.m_ui.g_navbar.set_style(gtk.TOOLBAR_BOTH)
                 self.set_string('gui/navbar_style', 'both')
-            if self.g_navbar_show.active:
-                self.m_app.m_ui.get_dock_item_by_name('backandforward').show()
+            if self.g_navbar_show.get_active():
+                self.m_app.m_ui.get_dock_item_by_name('navbar').show()
                 self.set_bool('gui/navbar_visible', 1)
             else:
-                self.m_app.m_ui.get_dock_item_by_name('backandforward').hide()
+                self.m_app.m_ui.get_dock_item_by_name('navbar').hide()
                 self.set_bool('gui/navbar_visible', 0)
             self.set_bool('gui/mainwin_user_resizeable',
-                          self.g_mainwin_user_resizeable.active)
-            if self.get_bool('gui/mainwin_user_resizeable'):
-                self.m_app.m_ui.set_policy(0, 1, 0)
-            else:
-                self.m_app.m_ui.set_policy(0, 0, 1)
+                          self.g_mainwin_user_resizeable.get_active())
+            self.m_app.m_ui.set_resizable(self.get_bool('gui/mainwin_user_resizeable'))
             self.set_string("config/web_browser", self.g_web_browser.get_text())
             self.set_string("config/mua", self.g_mail_program.get_text())
-            self.set_bool('gui/development_menu_visible',
-                          self.g_development_menu_visible.active)
-            if self.get_bool('gui/development_menu_visible'):
-                self.m_app.m_ui.get_dock_item_by_name("Menubar").children()[0].children()[1].show()
-            else:
-                self.m_app.m_ui.get_dock_item_by_name("Menubar").children()[0].children()[1].hide()
         if pagenum == 3:
             self.set_bool('config/picky_on_new_question',
-                          self.g_picky_on_new_question.active)
+                          self.g_picky_on_new_question.get_active())
             self.set_bool('config/auto_repeat_question_if_wrong_answer',
-                          self.g_autorepeat_if_wrong.active)
+                          self.g_autorepeat_if_wrong.get_active())
         if pagenum == 4:
             if soundcard.synth:
                 soundcard.synth.close()
-            if self.g_midiplayer_radio.active:
+            if self.g_midiplayer_radio.get_active():
                 soundcard.initialise_external_midiplayer(
                       self.g_midiplayer_entry.get_text())
-            elif self.g_device_radio.active:
+            elif self.g_device_radio.get_active():
                 if sys.platform != 'win32' and configureoutput.HAVE_LINUX_AWE_VOICE_H and \
-                    self.g_awe_checkbutton.active:
+                    self.g_awe_checkbutton.get_active():
                       card_info = "awe"
                 else:
                      card_info = ""
@@ -440,17 +433,16 @@ class ConfigWindow(gnome.ui.GnomePropertyBox, cfg.ConfigUtils):
                             card_info)
                 except (soundcard.SoundInitException, OSError), e:
                     self.m_app.display_sound_init_error_message(e)
-		    return
-            elif self.g_fakesynth_radio.active:
-                soundcard.initialise_using_fake_synth(0)
+                    return
             else:
-                raise "Shit"
-            if self.g_midiplayer_radio.active:
+                assert self.g_fakesynth_radio.get_active()
+                soundcard.initialise_using_fake_synth(0)
+            if self.g_midiplayer_radio.get_active():
                 self.set_string("sound/type", "external-midiplayer")
-            elif self.g_device_radio.active:
+            elif self.g_device_radio.get_active():
                 self.set_string("sound/type", "sequencer-device")
             else:
-                assert self.g_fakesynth_radio.active
+                assert self.g_fakesynth_radio.get_active()
                 self.set_string("sound/type", "fake-synth")
             if sys.platform != 'win32':
                 self.set_string("sound/device_file", self.g_device_entry.get_text())
@@ -464,5 +456,6 @@ class ConfigWindow(gnome.ui.GnomePropertyBox, cfg.ConfigUtils):
                             self.g_midiplayer_entry.get_text())
         if pagenum == -1:
             pass
-
+    def _watch_toggle_callback(self, widget, key):
+        widget.set_active(self.get_bool(key))
 
