@@ -1,5 +1,5 @@
 # GNU Solfege - eartraining for GNOME
-# Copyright (C) 2001  Tom Cato Amundsen
+# Copyright (C) 2001, 2002  Tom Cato Amundsen
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -18,8 +18,7 @@
 import sys, os, os.path
 if __name__ == "__main__":
     sys.path.append(".")
-import gettext
-from i18n import _
+import gettext, i18n
 import re, string
 tokens = (
  'NAME', 'EQUALS', 'STRING', 'I18NSTRING', 'INTEGER', 'LBRACE',
@@ -49,13 +48,13 @@ re_ALL = re.compile(r"""
             (?P<comment>\#.*?$)
             """, re.VERBOSE|re.MULTILINE|re.DOTALL)
 
-class DataparserException:
+class DataparserException(Exception):
     def __init__(self, filename, lineno, badcode):
         self.filename = filename
         self.lineno = lineno
         self.badcode = badcode
     def __str__(self):
-        return "dataparser-error, lineno: %i\nThis code is invalid: %s: %s" % (self.filename, self.lineno, self.badcode)
+        return "Failed to parse lessonfile '%s'.\nError at line %s." % (self.filename, self.lineno)
 
 
 class Dataparser:
@@ -101,7 +100,12 @@ class Dataparser:
                 # music = "this is a string"
                 context['music'] = self.evaluate(ex)
             else:
-                raise "Parse error in '%s', line %i" % (ex[2], ex[3]), ex[1]
+                #FIXME are there other places where ex is not as simple
+                # as we think.
+                if len(ex) > 3:
+                    raise DataparserException(ex[2], ex[3], ex[1])
+                else:
+                    raise DataparserException(ex[1][2], ex[1][3], ex[1][1])
     def get_tokens(self, filename, first_block_only):
         f = open(filename, "r")
         s = f.read()
@@ -194,9 +198,9 @@ class Dataparser:
                                       int(m.group('rat_den')))))
                     continue
                 else:
-                    raise "unmatched", m.groups()
+                    raise DataparserException(filename, linecount, s[p:p+10])
             else:
-                raise "miss '%s'" % s[p:p+10]
+                raise DataparserException(filename, linecount, s[p:p+10])
         return v
     def do_include(self, list):
         x = 0
@@ -250,13 +254,12 @@ class Dataparser:
                 return [A, B]
         elif exp[0] == NAME:
             return self.m_globals[exp[1]]
-        elif exp[0] == TEMPO:
-            return exp[1]
         else:
-            raise "unfinished", exp
+            assert exp[0] == TEMPO
+            return exp[1]
 
 def get_translated_string(dict, name):
-    for n in gettext.lang:
+    for n in i18n.langs():
         if dict.has_key("%s(%s)" % (name, n)):
             return dict["%s(%s)" % (name, n)]
     return dict[name]
@@ -274,6 +277,7 @@ def __f():
                       'progression': 'progression',
                       'harmony': 'harmony',
                       'sing-chord': 'sing-chord',
+                      'chord-voicing': 'chord-voicing',
                       'chord': 'chord',
                       'id-by-name': 'id-by-name',
                       'satb': 'satb',
@@ -284,7 +288,8 @@ def __f():
                       'tempo': (160, 4)})
         for x in range(10):
             p.parse_file(fn)
-       #print p.m_blocks
+        for b in p.m_blocks:
+            print b
     print time.clock() - t1
 
 def _benchmark_tokenizer():
@@ -301,8 +306,6 @@ def _benchmark_tokenizer():
 
 if __name__ == "__main__":
     import i18n
-    i18n.initialise("share/locale")
-    from i18n import _
     #print _benchmark_tokenizer()
     __f()
     #import profile, pstats
