@@ -1,5 +1,5 @@
 # GNU Solfege - eartraining for GNOME
-# Copyright (C) 2000, 2001, 2002  Tom Cato Amundsen
+# Copyright (C) 2000, 2001, 2002, 2003  Tom Cato Amundsen
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -16,9 +16,8 @@
 # Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 
 import gtk
-import gnome.uiconsts
+import gnome
 import soundcard
-from i18n import _
 import cfg
 
 #  Prefixes used in this module:
@@ -28,28 +27,28 @@ import cfg
 
 
 def tLabel(table, x1, x2, y1, y2, text="", xalign=0.0, yalign=0.5, xoptions=gtk.EXPAND|gtk.FILL, yoptions=gtk.EXPAND|gtk.FILL, xpadding=0, ypadding=0):
-    label = gtk.GtkLabel(text)
+    label = gtk.Label(text)
     label.set_alignment(xalign, yalign)
     table.attach(label, x1, x2, y1, y2, xoptions=xoptions, yoptions=yoptions, xpadding=xpadding, ypadding=ypadding)
     return label
 
 def bLabel(pack_into, label, expand=gtk.TRUE, fill=gtk.TRUE):
-    b = gtk.GtkLabel(label)
+    b = gtk.Label(label)
     b.show()
     pack_into.pack_start(b, expand, fill)
     return b
 
 def bButton(pack_into, label, callback=None, expand=gtk.TRUE, fill=gtk.TRUE):
-    b = gtk.GtkButton(label)
+    b = gtk.Button(label)
     b.show()
     if callback:
         b.connect('clicked', callback)
     pack_into.pack_start(b, expand, fill)
     return b
 
-class nSpinButton(gtk.GtkSpinButton, cfg.ConfigUtils):#FIXME (??what is there to fix???)
+class nSpinButton(gtk.SpinButton, cfg.ConfigUtils):#FIXME (??what is there to fix???)
     def __init__(self, exname, name, adj, climb_rate=1, digits=1):
-        gtk.GtkSpinButton.__init__(self, adj, climb_rate, digits)
+        gtk.SpinButton.__init__(self, adj, climb_rate, digits)
         cfg.ConfigUtils.__init__(self, exname)
         self.m_name = name
         self.set_digits(0)
@@ -57,39 +56,43 @@ class nSpinButton(gtk.GtkSpinButton, cfg.ConfigUtils):#FIXME (??what is there to
         self.set_value(self.get_float(self.m_name))
         if self.get_value() != self.get_float(self.m_name):
             self.set_float(self.m_name, self.get_value())
-        self.connect('changed', self.on_changed)
+        self.connect('value-changed', self.on_changed)
         self._watch_id = self.add_watch(self.m_name, self._watch_cb)
+        self.m_stop_watch = 0
     def _watch_cb(self, name):
-        gtk.GtkSpinButton.set_value(self, self.get_float(name))
+        if not self.m_stop_watch:
+            gtk.SpinButton.set_value(self, self.get_float(name))
     def set_value(self, value):
-        gtk.GtkSpinButton.set_value(self, value)
+        gtk.SpinButton.set_value(self, value)
         self.set_float(self.m_name, value)
     def on_changed(self, _o):
+        self.m_stop_watch = 1
         self.set_float(self.m_name, self.get_value())
+        self.m_stop_watch = 0
 
 def tSpinButton(table, x1, x2, y1, y2,
                 value, lower, upper, step_incr=1, page_incr=10, callback=None):
-    adj = gtk.GtkAdjustment(value, lower, upper, step_incr, page_incr)
-    spin = gtk.GtkSpinButton(adj, digits=0)
+    adj = gtk.Adjustment(value, lower, upper, step_incr, page_incr)
+    spin = gtk.SpinButton(adj, digits=0)
     if callback:
-        spin.connect('changed', callback)
+        spin.connect('value-changed', callback)
     table.attach(spin, x1, x2, y1, y2)
     return spin
 
 def bHBox(pack_into, expand=gtk.TRUE, fill=gtk.TRUE, padding=0):
-    b = gtk.GtkHBox()
+    b = gtk.HBox()
     b.show()
     pack_into.pack_start(b, expand, fill, padding)
     return b
 
 def bVBox(pack_into, expand=gtk.TRUE, fill=gtk.TRUE, padding=0):
-    b = gtk.GtkVBox()
+    b = gtk.VBox()
     pack_into.pack_start(b, expand, fill, padding)
     return b
 
-class nCheckButton(gtk.GtkCheckButton, cfg.ConfigUtils):
+class nCheckButton(gtk.CheckButton, cfg.ConfigUtils):
     def __init__(self, exname, name, label=None, default_value=0, callback=None):
-        gtk.GtkCheckButton.__init__(self, label)
+        gtk.CheckButton.__init__(self, label)
         #cfg.ConfigUtils.__init__(self, exname)
         cfg.ConfigUtils.__dict__['__init__'](self, exname)
         self.m_name = name
@@ -112,13 +115,13 @@ class nCheckButton(gtk.GtkCheckButton, cfg.ConfigUtils):
             self.m_callback(_o)
  
 def RadioButton(group, label, callback=None):
-    rdb = gtk.GtkRadioButton(group, label)
+    rdb = gtk.RadioButton(group, label)
     if callback:
         rdb.connect('toggled', callback)
     rdb.show()
     return rdb
 
-class nCombo(gtk.GtkCombo, cfg.ConfigUtils):
+class nCombo(gtk.Combo, cfg.ConfigUtils):
     def __init__(self, exname, name, default, popdown_strings):
         """
         Be aware that the value of the entry, is stored as an integer
@@ -130,7 +133,7 @@ class nCombo(gtk.GtkCombo, cfg.ConfigUtils):
         the actual value of the entry, we get into trouble when running
         the program with other locale settings.
         """
-        gtk.GtkCombo.__init__(self)
+        gtk.Combo.__init__(self)
         #cfg.ConfigUtils.__init__(self, exname)
         cfg.ConfigUtils.__dict__['__init__'](self, exname)
         self.popdown_strings = popdown_strings
@@ -147,55 +150,32 @@ class nCombo(gtk.GtkCombo, cfg.ConfigUtils):
     def entry_changed(self, entry):
         self.set_int(self.m_name, self.popdown_strings.index(entry.get_text()))
 
-class PercussionNameCombo(gtk.GtkCombo, cfg.ConfigUtils):
+class PercussionNameOptionMenu(gtk.OptionMenu, cfg.ConfigUtils):
     def __init__(self, exname, name, default):
-        gtk.GtkCombo.__init__(self)
+        gtk.OptionMenu.__init__(self)
         cfg.ConfigUtils.__init__(self, exname)
         self.m_name = name
-        self.set_value_in_list(gtk.TRUE, gtk.FALSE)
-        self.set_popdown_strings(soundcard.percussion_names)
+        menu = gtk.Menu()
+        for pn in soundcard.percussion_names:
+            menu.add(gtk.MenuItem(pn))
+        self.set_menu(menu)
         i = self.get_int(name)
         if not i:
             i = soundcard.percussionname_to_int(default)
             self.set_int(name, i)
-        self.entry.set_text(soundcard.int_to_percussionname(i))
-        self.entry.connect("changed", self.entry_changed)
-        self.entry.set_editable(gtk.FALSE)
-        self.show()
-    def entry_changed(self, entry):
-        self.set_int(self.m_name,
-                     soundcard.percussionname_to_int(entry.get_text()))
-
-class SolfegeDialog(gtk.GtkDialog):
-    def __init__(self, heading, txt, buttons=(), callback=None):
-        gtk.GtkDialog.__init__(self)
-        self.m_callback = callback
-        self.vbox.set_border_width(gnome.uiconsts.PAD)
-        self.connect('delete_event', self.destroy)
-
-        label = bLabel(self.vbox, heading)
-        label.set_name('Heading1')
-
-        label = bLabel(self.vbox, txt)
-        label.set_justify(gtk.JUSTIFY_LEFT)
-        for text in buttons:
-            b = gtk.GtkButton(text)
-            b.set_data('user_data', text)
-            b.connect('clicked', self.click)
-            self.action_area.pack_start(b)
-            b.show()
-    def click(self, button):
-        if self.m_callback:
-            self.m_callback(button.get_data('user_data'))
-        self.destroy()
+        self.set_history(i)
+        self.connect("changed", self.entry_changed)
+    def entry_changed(self, widget):
+        #UGH value 35 should be taken from soundcard module.
+        self.set_int(self.m_name, widget.get_history()+35)
 
 
-class FlashBar(gtk.GtkFrame):
+class FlashBar(gtk.Frame):
     def __init__(self):
-        gtk.GtkFrame.__init__(self)
+        gtk.Frame.__init__(self)
         self.set_shadow_type(gtk.SHADOW_IN)
-        #UGH
-        self.set_usize(358, 40)
+        #FIXME different gtk themes can make these values wrong
+        self.set_size_request(358, 40)
         self.__stack = []
         import widgets
         self.__label = widgets.HarmonicProgressionLabel('')
@@ -227,10 +207,13 @@ class FlashBar(gtk.GtkFrame):
         """If a message is being flashed right now, that flashing is
         not affected, but the message below the flashed message is removed.
         """
-        self.__stack.pop()
+        if self.__stack:
+            self.__stack.pop()
         if not self.__timeout:
             if self.__stack:
                 self.__label.set_text(self.__stack[-1])
+            else:
+                self.__label.set_text('')
     def clear(self):
         self.__stack = []
         if not self.__timeout:
