@@ -1,5 +1,5 @@
 # GNU Solfege - eartraining for GNOME
-# Copyright (C) 2000, 2001, 2002  Tom Cato Amundsen
+# Copyright (C) 2000, 2001, 2002, 2003  Tom Cato Amundsen
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -21,42 +21,45 @@ import abstract, const, mpd
 import statistics, statisticsviewer
 import lessonfile
 import string
-from i18n import _
+import dataparser
+import soundcard
 
 
 class Teacher(abstract.LessonbasedTeacher):
     OK = 0
     ERR_PICKY = 1
-    ERR_NO_LESSONFILE = 2
     def __init__(self, exname,  app, use=const.USE_ID_BY_NAME):
         abstract.LessonbasedTeacher.__init__(self, exname, app, use)
         self.lessonfileclass = lessonfile.IdByNameLessonfile
-        self.parse_lessonfile()
-
+        try: # the lessonfile might contain errors.
+            self.parse_lessonfile()
+        except dataparser.DataparserException, e:
+            self.m_app.m_ui.display_error_message(str(e))
         self.m_statistics = statistics.LessonStatistics(self)
     def give_up(self):
         self.q_status = const.QSTATUS_GIVE_UP
     def new_question(self):
+        """
+        UI will never call this function unless we have a usable lessonfile.
+        """
         if self.m_timeout_handle:
             gtk.timeout_remove(self.m_timeout_handle)
             self.m_timeout_handle = None
- 
+
         if self.get_bool('config/picky_on_new_question') \
                  and self.q_status in [const.QSTATUS_NEW, const.QSTATUS_WRONG]:
              return Teacher.ERR_PICKY
 
         self.q_status = const.QSTATUS_NO
 
-        if self.m_P:
-            self.m_P.select_random_question()
-            self.q_status = const.QSTATUS_NEW
-            return self.OK
-        else:
-            return self.ERR_NO_LESSONFILE
+        assert self.m_P
+        self.m_P.select_random_question()
+        self.q_status = const.QSTATUS_NEW
+        return self.OK
     def guess_answer(self, answer):
         """
         Return: 1 if correct, None if wrong
-        """        
+        """
         if answer == self.m_P.get_untranslated_name():
             if self.q_status == const.QSTATUS_NEW:
                 self.m_statistics.add_correct(answer)
@@ -68,39 +71,35 @@ class Teacher(abstract.LessonbasedTeacher):
                 self.m_statistics.add_wrong(self.m_P.get_untranslated_name(),
                                             answer)
                 self.q_status = const.QSTATUS_WRONG
-        
+
 
 class Gui(abstract.LessonbasedGui):
     def __init__(self, teacher, window):
         abstract.LessonbasedGui.__init__(self, teacher, window)
-        self.m_key_bindings = {'new_ak': self.new_question,
-                               'repeat_ak': self.m_t.play_question,
-                               'give_up_ak': self.give_up,
-                               'show_ak': self.show_answer}
         ################
         # practise_box #
         ################
         self.g_bb = widgets.QuestionNameButtonTable()
         self.practise_box.pack_start(self.g_bb, gtk.FALSE)
-        
+
         self.g_flashbar = gu.FlashBar()
         self.g_flashbar.show()
         self.practise_box.pack_start(self.g_flashbar, gtk.FALSE)
-        self.practise_box.set_spacing(gnome.uiconsts.PAD)
+        self.practise_box.set_spacing(gnome.ui.PAD)
 
-        self.g_new = gu.bButton(self.action_area, _("New"), self.new_question)
-        self.g_repeat = gu.bButton(self.action_area, _("Repeat"),
+        self.g_new = gu.bButton(self.action_area, _("_New"), self.new_question)
+        self.g_repeat = gu.bButton(self.action_area, _("_Repeat"),
                                    lambda _o, self=self: self.m_t.play_question())
-        self.g_give_up = gu.bButton(self.action_area, _("Give up"), self.give_up)
+        self.g_give_up = gu.bButton(self.action_area, _("_Give up"), self.give_up)
         self.g_give_up.set_sensitive(gtk.FALSE)
 
-        self.g_show = gu.bButton(self.action_area, _("Show"), self.show_answer)
+        self.g_show = gu.bButton(self.action_area, _("_Show"), self.show_answer)
         self.g_show.set_sensitive(gtk.FALSE)
         self.practise_box.show_all()
         ##############
         # config_box #
         ##############
-        self.config_box.set_spacing(gnome.uiconsts.PAD_SMALL)
+        self.config_box.set_spacing(gnome.ui.PAD_SMALL)
         self.g_lessonfile_selector = widgets.SelectLessonfileWidget(
             self.m_t, self.g_win, self.update_gui_after_lessonfile_change)
         self.g_lessonfile_selector.show()
@@ -143,6 +142,7 @@ class Gui(abstract.LessonbasedGui):
             self.m_t.give_up()
             self.g_show.set_sensitive(gtk.TRUE)
             self.g_new.set_sensitive(gtk.TRUE)
+            self.g_new.grab_focus()
             self.g_give_up.set_sensitive(gtk.FALSE)
     def on_click(self, button):
         if self.m_t.q_status == const.QSTATUS_NO:
@@ -157,6 +157,7 @@ class Gui(abstract.LessonbasedGui):
             if self.m_t.guess_answer(button.get_data('pcode')):
                 self.g_flashbar.flash(_("Correct"))
                 self.g_new.set_sensitive(gtk.TRUE)
+                self.g_new.grab_focus()
                 self.g_show.set_sensitive(gtk.TRUE)
                 self.g_give_up.set_sensitive(gtk.FALSE)
             else:
@@ -165,23 +166,39 @@ class Gui(abstract.LessonbasedGui):
                     self.m_t.play_question()
                 self.g_give_up.set_sensitive(gtk.TRUE)
     def new_question(self, widget=None):
+        # if we have no lessonfile, then we have to questions.
+        if not self.m_t.m_P:
+            return
         g = self.m_t.new_question()
         if g == self.m_t.OK:
-            self.g_repeat.set_sensitive(gtk.TRUE)
-            self.g_new.set_sensitive(
-                not self.get_bool('config/picky_on_new_question'))
-            self.g_give_up.set_sensitive(gtk.FALSE)
-            self.g_show.set_sensitive(gtk.FALSE)
-            self.g_flashbar.clear()
-            self.m_t.play_question()
+            try:
+                self.m_t.play_question()
+            except mpd.MpdException, e:
+                # since we could not play the new question, it is possible,
+                # if the user are fast, that the last question is still playing.
+                soundcard.synth.stop()
+                self.g_win.display_question_music_error_message(
+                    self.m_t.m_P._idx, self.get_string('lessonfile'), e)
+                self.m_t.q_status = const.QSTATUS_NO
+                self.g_repeat.set_sensitive(gtk.FALSE)
+                self.g_show.set_sensitive(gtk.FALSE)
+            else:
+                self.g_repeat.set_sensitive(gtk.TRUE)
+                self.g_new.set_sensitive(
+                    not self.get_bool('config/picky_on_new_question'))
+                self.g_bb.grab_focus_first_button()
+                self.g_give_up.set_sensitive(gtk.FALSE)
+                self.g_show.set_sensitive(gtk.FALSE)
+                self.g_flashbar.clear()
     def on_start_practise(self):
+        self.g_new.grab_focus()
         self.m_t.m_statistics.reset_session()
         gtk.timeout_add(const.SHORT_WAIT, lambda self=self:
             self.g_flashbar.flash(_("Click 'New' to begin.")))
     def on_end_practise(self):
+        self.m_t.end_practise()
         self.g_repeat.set_sensitive(gtk.FALSE)
         self.g_show.set_sensitive(gtk.FALSE)
         self.g_new.set_sensitive(gtk.TRUE)
         self.g_give_up.set_sensitive(gtk.FALSE)
         self.g_flashbar.clear()
-        self.m_t.end_practise()
