@@ -1,5 +1,5 @@
 # GNU Solfege - eartraining for GNOME
-# Copyright (C) 2000, 2001, 2002  Tom Cato Amundsen
+# Copyright (C) 2000, 2001, 2002, 2003  Tom Cato Amundsen
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -21,7 +21,6 @@ import gtk, gnome
 import abstract, gu, const, mpd, mpd.musicdisplayer
 import utils
 import statistics, statisticsviewer
-from i18n import _
 
 scaledata = {
     'ionian': ('[c8 d][e f][g a][b c]|[c b][a g][f e][d c]',
@@ -57,7 +56,10 @@ class Teacher(abstract.Teacher):
     def give_up(self):
         self.q_status = const.QSTATUS_GIVE_UP
     def new_scale(self):
-        """Return 1 if we have a new question, otherwise None
+        """Return values:
+        OK
+        ERR_NO_SCALES
+        ERR_PICKY
         """
         if self.m_timeout_handle:
             gtk.timeout_remove(self.m_timeout_handle)
@@ -80,7 +82,7 @@ class Teacher(abstract.Teacher):
     def get_mpd_string(self):
         assert self._scale
         return r"\staff\transpose %s\relative c'{ \clef violin \key %s \major %s }" \
-                   % (self.m_tonika.str(), scaledata[self._scale][2], scaledata[self._scale][0])
+                   % (self.m_tonika.get_octave_notename(), scaledata[self._scale][2], scaledata[self._scale][0])
     def play(self):
         if self.q_status == const.QSTATUS_NO:
             return
@@ -128,77 +130,70 @@ class Teacher(abstract.Teacher):
 class Gui(abstract.Gui):
     def __init__(self, teacher, window):
         abstract.Gui.__init__(self, teacher, window)
-        self.m_key_bindings = {
-           'new_ak': self.new_question,
-           'repeat_ak': self.m_t.play,
-           'repeat_slowly_ak': self.m_t.play_slowly,
-           'give_up_ak': self.give_up}
         ################
         # practise_box #
         ################
         self.g_score_displayer = mpd.musicdisplayer.MusicDisplayer(utils.play_tone)
-        self.g_score_displayer.set_usize(-1, 150)
         self.practise_box.pack_start(self.g_score_displayer)
 
         self.g_flashbar = gu.FlashBar()
         self.g_flashbar.show()
         self.practise_box.pack_start(self.g_flashbar, gtk.FALSE)
-        self.practise_box.set_spacing(gnome.uiconsts.PAD)
+        self.practise_box.set_spacing(gnome.ui.PAD)
 
-        self.table = gtk.GtkTable(3, 3, gtk.TRUE)
+        self.table = gtk.Table(3, 3, gtk.TRUE)
         self.practise_box.pack_start(self.table, gtk.FALSE)
-        self.g_scale_checkbuttons = {}
+        self.g_scale_buttons = {}
         for k in scaledata.keys():
-            b = gtk.GtkButton(scaledata[k][1])
+            self.g_scale_buttons[k] = b = gtk.Button(scaledata[k][1])
             b.connect('clicked', self.on_scale_click, k)
             self.table.attach(b, scaledata[k][3], scaledata[k][3]+1,
                               scaledata[k][4], scaledata[k][4]+1)
-        self.g_new = gu.bButton(self.action_area, _("New scale"),
+        self.g_new = gu.bButton(self.action_area, _("_New scale"),
                                    self.new_question)
-        self.g_repeat = gu.bButton(self.action_area, _("Repeat scale"),
+        self.g_repeat = gu.bButton(self.action_area, _("_Repeat"),
                  lambda _o, self=self: self.m_t.play())
         self.g_repeat.set_sensitive(gtk.FALSE)
-        self.g_repeat_slowly = gu.bButton(self.action_area, _("Repeat slowly"),
+        self.g_repeat_slowly = gu.bButton(self.action_area, _("Repeat s_lowly"),
                  lambda _o, self=self: self.m_t.play_slowly())
         self.g_repeat_slowly.set_sensitive(gtk.FALSE)
-        self.g_give_up = gu.bButton(self.action_area, _("Give up"), self.give_up)
+        self.g_give_up = gu.bButton(self.action_area, _("_Give up"), self.give_up)
         self.g_give_up.set_sensitive(gtk.FALSE)
-        self.practise_box.set_spacing(gnome.uiconsts.PAD)
+        self.practise_box.set_spacing(gnome.ui.PAD)
         self.practise_box.show_all()
         ##############
         # config_box #
         ##############
-        self.config_box.set_spacing(gnome.uiconsts.PAD)
-        frame = gtk.GtkFrame(_("Ask for these scales:"))
+        self.config_box.set_spacing(gnome.ui.PAD)
+        frame = gtk.Frame(_("Ask for these scales"))
         self.config_box.pack_start(frame, gtk.FALSE)
-        table = gtk.GtkTable(4, 3)
-        table.set_border_width(gnome.uiconsts.PAD_SMALL)
+        table = gtk.Table(4, 3)
+        table.set_border_width(gnome.ui.PAD_SMALL)
         frame.add(table)
         for k in scaledata.keys():
             b = gu.nCheckButton(self.m_exname, k, scaledata[k][1], default_value=1)
             table.attach(b, scaledata[k][3], scaledata[k][3]+1,
                          scaledata[k][4], scaledata[k][4]+1)
-        frame = gtk.GtkFrame(_("Override global settings"))
+        frame = gtk.Frame()
         self.config_box.pack_start(frame, gtk.FALSE)
-        vbox = gtk.GtkVBox()
-        vbox.set_border_width(gnome.uiconsts.PAD)
+        vbox = gtk.VBox()
+        vbox.set_border_width(gnome.ui.PAD)
         frame.add(vbox)
         self.g_override_global_settings = \
                gu.nCheckButton(self.m_exname, 'override_global_settings',
-                            _("Enable"), callback=self.update_config_override)
-        vbox.pack_start(self.g_override_global_settings, gtk.FALSE)
-        self.mt = table = gtk.GtkTable()
-        table.set_col_spacings(50)
+                            _("Override global settings"), callback=self.update_config_override)
+        frame.set_label_widget(self.g_override_global_settings)
+        self.mt = table = gtk.Table()
         vbox.pack_start(table, gtk.FALSE)
         gu.tLabel(table, 0, 1, 0, 1, _("Slow bpm:"),
-                  xalign=1.0, xoptions=gtk.FILL, xpadding=gnome.uiconsts.PAD)
+                  xalign=1.0, xoptions=gtk.FILL, xpadding=gnome.ui.PAD)
         gu.tLabel(table, 0, 1, 1, 2, _("Default bpm:"),
-                  xalign=1.0, xoptions=gtk.FILL, xpadding=gnome.uiconsts.PAD)
+                  xalign=1.0, xoptions=gtk.FILL, xpadding=gnome.ui.PAD)
         table.attach(gu.nSpinButton(self.m_exname, 'slow_bpm',
-                               gtk.GtkAdjustment(0, 10, 255, 1, 10), digits=0),
+                               gtk.Adjustment(0, 10, 255, 1, 10), digits=0),
                      1, 2, 0, 1, xoptions=0)
         table.attach(gu.nSpinButton(self.m_exname, 'default_bpm',
-                               gtk.GtkAdjustment(0, 10, 255, 1, 10), digits=0),
+                               gtk.Adjustment(0, 10, 255, 1, 10), digits=0),
                      1, 2, 1, 2, xoptions=0)
         self.update_config_override()
         # ------------------------------------------
@@ -212,7 +207,7 @@ class Gui(abstract.Gui):
         self.setup_statisticsviewer(statisticsviewer.StatisticsViewer,
                                    _("Identify scale"))
     def update_config_override(self, _o=None):
-        if self.g_override_global_settings.active:
+        if self.g_override_global_settings.get_active():
             self.mt.set_sensitive(gtk.TRUE)
         else:
             self.mt.set_sensitive(gtk.FALSE)
@@ -222,6 +217,7 @@ class Gui(abstract.Gui):
             self.g_score_displayer.display(self.m_t.get_mpd_string(), fontsize)
             self.m_t.give_up()
             self.g_new.set_sensitive(gtk.TRUE)
+            self.g_new.grab_focus()
             self.g_give_up.set_sensitive(gtk.FALSE)
             self.g_flashbar.push(_("The answer is: %s") % self.m_t.scale_key_to_name(self.m_t._scale))
     def new_question(self, widget=None):
@@ -234,6 +230,7 @@ class Gui(abstract.Gui):
             self.g_give_up.set_sensitive(gtk.FALSE)
             self.g_new.set_sensitive(
                      not self.get_bool('config/picky_on_new_question'))
+            self.g_scale_buttons['ionian'].grab_focus()
             self.g_flashbar.clear()
         elif g == Teacher.ERR_NO_SCALES:
             self.g_flashbar.flash(_("You have to select some scales to practise."))
@@ -241,22 +238,22 @@ class Gui(abstract.Gui):
             self.g_repeat_slowly.set_sensitive(gtk.FALSE)
             self.g_repeat.set_sensitive(gtk.FALSE)
             self.g_flashbar.clear()
-        elif g == Teacher.ERR_PICKY:
-            self.g_flashbar.flash(_("You have to solve this question first."))
         else:
-            raise "NEVER REACH THIS!"
+            assert g == Teacher.ERR_PICKY
+            self.g_flashbar.flash(_("You have to solve this question first."))
     def on_start_practise(self):
         self.m_t.m_statistics.reset_session()
+        self.g_new.grab_focus()
         self.g_score_displayer.clear()
         gtk.timeout_add(const.SHORT_WAIT, lambda self=self:
            self.g_flashbar.flash(_("Click 'New scale' to begin.")))
     def on_end_practise(self):
+        self.m_t.end_practise()
         self.g_new.set_sensitive(gtk.TRUE)
         self.g_repeat.set_sensitive(gtk.FALSE)
         self.g_repeat_slowly.set_sensitive(gtk.FALSE)
         self.g_give_up.set_sensitive(gtk.FALSE)
         self.g_flashbar.clear()
-        self.m_t.end_practise()
     def on_scale_click(self, button, name):
         if self.m_t.q_status == const.QSTATUS_GIVE_UP:
             return
@@ -272,7 +269,10 @@ class Gui(abstract.Gui):
             if self.m_t.guess_answer(name):
                 self.g_flashbar.flash(_("Correct"))
                 self.g_new.set_sensitive(gtk.TRUE)
+                self.g_new.grab_focus()
                 self.g_give_up.set_sensitive(gtk.FALSE)
+                fontsize = self.get_int('config/feta_font_size=20')
+                self.g_score_displayer.display(self.m_t.get_mpd_string(), fontsize)
             else:
                 self.g_flashbar.flash(_("Wrong"))
                 if self.get_bool("config/auto_repeat_question_if_wrong_answer"):
