@@ -1,5 +1,5 @@
 # GNU Solfege - eartraining for GNOME
-# Copyright (C) 2000, 2001, 2002  Tom Cato Amundsen
+# Copyright (C) 2000, 2001, 2002, 2003  Tom Cato Amundsen
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -17,7 +17,6 @@
 
 import inputwidgets, abstract, const, random, soundcard, gu, mpd, gnome
 import configureoutput
-from i18n import _
 import gtk
 import gu
 import statistics, statisticsviewer
@@ -75,7 +74,7 @@ class Teacher(abstract.Teacher):
         else:
             if self.q_status == const.QSTATUS_NEW:
                 self.m_statistics.add_wrong(self.m_question, notename)
-            self.q_status = const.QSTATUS_WRONG
+                self.q_status = const.QSTATUS_WRONG
     def play_question(self):
         if self.q_status == const.QSTATUS_NO:
             return
@@ -83,6 +82,7 @@ class Teacher(abstract.Teacher):
              mpd.notename_to_int(self.m_question)+self.m_octave*12,
              self.get_int('config/preferred_instrument_velocity'))
     def give_up(self):
+        self.m_statistics.reset_session()
         self.q_status = const.QSTATUS_GIVE_UP
     def spank_me_play_question(self):
         track = soundcard.Track()
@@ -97,16 +97,13 @@ class Gui(abstract.Gui):
     def __init__(self, teacher, window):
         abstract.Gui.__init__(self, teacher, window)
         if not configureoutput.WITH_GNOME:
-            self.practise_box.add(gtk.GtkLabel(_(
+            self.practise_box.add(gtk.Label(_(
 """This exercise don't work without GNOME.
-It can pretty easy be converted to use GtkPixmap instead
+It can pretty easy be converted to use Pixmap instead
 of GnomeCanvas, but I (tca) don't have time to do this
 right now. But I can answer questions if needed.""")))
             self.practise_box.show_all()
             return
-        self.m_key_bindings = {'new_ak': self.new_question,
-                               'repeat_ak': self.m_t.play_question,
-                               'give_up_ak': self.give_up}
         v = []
         for k in ('c', 'cis', 'd', 'dis', 'e', 'f', 'fis',
                   'g', 'gis', 'a', 'ais', 'b'):
@@ -123,46 +120,46 @@ right now. But I can answer questions if needed.""")))
         self.g_flashbar = gu.FlashBar()
         self.g_flashbar.show()
         self.practise_box.pack_start(self.g_flashbar, gtk.FALSE)
-        self.practise_box.set_spacing(gnome.uiconsts.PAD)
+        self.practise_box.set_spacing(gnome.ui.PAD)
 
-        self.g_new_tone = gu.bButton(self.action_area, _("New tone"),
+        self.g_new_tone = gu.bButton(self.action_area, _("_New tone"),
                                      self.new_question)
-        self.g_repeat = gu.bButton(self.action_area, _("Repeat"),
+        self.g_repeat = gu.bButton(self.action_area, _("_Repeat"),
                   lambda _o, self=self: self.m_t.play_question())
         self.g_repeat.set_sensitive(gtk.FALSE)
-        self.g_give_up = gu.bButton(self.action_area, _("Give up"), self.give_up)
+        self.g_give_up = gu.bButton(self.action_area, _("_Give up"), self.give_up)
         self.g_give_up.set_sensitive(gtk.FALSE)
         self.practise_box.show_all()
         ##############
         # config_box #
         ##############
-        self.config_box.set_spacing(gnome.uiconsts.PAD_SMALL)
-        table = gtk.GtkTable()
-        table.set_border_width(gnome.uiconsts.PAD_SMALL)
-        frame = gtk.GtkFrame(_("Weight"))
+        self.config_box.set_spacing(gnome.ui.PAD_SMALL)
+        table = gtk.Table()
+        table.set_border_width(gnome.ui.PAD_SMALL)
+        frame = gtk.Frame(_("Weight"))
         self.config_box.pack_start(frame, gtk.FALSE)
         frame.add(table)
         for x, n in [(1, 'cis'), (3, 'dis'), (7, 'fis'),
                      (9, 'gis'), (11, 'ais')]:
-             label = gtk.GtkLabel(n)
+             label = gtk.Label(mpd.MusicalPitch.new_from_notename(n).get_user_notename())
              label.set_name("Heading2")
              label.set_alignment(0.2, 1.0)
              table.attach(label, x, x+2, 0, 1, xoptions=gtk.FILL)
              b = gu.nSpinButton(self.m_exname, n+"_weight",
-                      gtk.GtkAdjustment(1, 0, 1000, 1, 10), digits=0)
+                      gtk.Adjustment(1, 0, 1000, 1, 10), digits=0)
              table.attach(b, x, x+2, 1, 2, xoptions=gtk.FILL)
         for x, n in [(0, 'c'), (2, 'd'), (4, 'e'), (6, 'f'),
                       (8, 'g'), (10, 'a'), (12, 'b')]:
-             label = gtk.GtkLabel(n)
+             label = gtk.Label(n)
              label.set_name("Heading2")
              label.set_alignment(0.35, 1.0)
              table.attach(label, x, x+2, 2, 3, xoptions=gtk.FILL)
              b = gu.nSpinButton(self.m_exname, n+"_weight",
-                   gtk.GtkAdjustment(1, 0, 1000, 1, 10), digits=0)
+                   gtk.Adjustment(1, 0, 1000, 1, 10), digits=0)
              table.attach(b, x, x+2, 3, 4, xoptions=gtk.FILL)
 
         hbox = gu.bHBox(self.config_box, gtk.FALSE)
-        hbox.pack_start(gtk.GtkLabel(_("Octave:")), gtk.FALSE, padding=4)
+        hbox.pack_start(gtk.Label(_("Octave:")), gtk.FALSE, padding=4)
         for oct in self.m_t.OCTAVES:
             b = gu.nCheckButton(self.m_exname, "octave"+str(oct), str(oct),
                                 default_value=1)
@@ -170,9 +167,9 @@ right now. But I can answer questions if needed.""")))
         #############
         self._add_auto_new_question_gui(self.config_box)
         #############
-        frame = gtk.GtkFrame(_("When you guess wrong"))
-        vbox = gtk.GtkVBox()
-        vbox.set_border_width(gnome.uiconsts.PAD_SMALL)
+        frame = gtk.Frame(_("When you guess wrong"))
+        vbox = gtk.VBox()
+        vbox.set_border_width(gnome.ui.PAD_SMALL)
         frame.add(vbox)
         vbox.pack_start(gu.nCheckButton(self.m_exname,
                     "warning_sound", _("Play warning sound")))
@@ -189,13 +186,14 @@ right now. But I can answer questions if needed.""")))
         s = self.m_t.q_status
         g = self.m_t.new_question()
         if g == Teacher.ERR_CONFIG:
-            gnome.ui.GnomeErrorDialog("You have to select the tones to use")
+            self.g_win.display_error_message(
+_("""You have to select some tones practise. Do this on the config page by setting the weight of tones to a value greater than zero."""))
             return
         elif g == Teacher.OK:
             self.g_new_tone.set_sensitive(
                   not self.get_bool('config/picky_on_new_question'))
             self.g_repeat.set_sensitive(gtk.TRUE)
-            if s == const.QSTATUS_NO:
+            if s in (const.QSTATUS_NO, const.QSTATUS_GIVE_UP):
                 self.g_flashbar.flash(_("First tone is %s")
                                       % self.m_t.m_question)
             self.g_flashbar.clear()
@@ -240,6 +238,7 @@ right now. But I can answer questions if needed.""")))
                 % self.m_t.m_question)
             self.m_t.give_up()
             self.g_new_tone.set_sensitive(gtk.TRUE)
+            self.g_new_tone.grab_focus()
             self.g_give_up.set_sensitive(gtk.FALSE)
     def set_percentage_label(self):
         self.g_percentage.set_text("%.1f %%" % (self.m_t.m_statistics.get()*100))
@@ -251,9 +250,11 @@ right now. But I can answer questions if needed.""")))
         gtk.timeout_add(const.SHORT_WAIT, lambda self=self:
             self.g_flashbar.flash(_("Click 'New tone' to begin.")))
         self.g_new_tone.set_sensitive(gtk.TRUE)
+        self.g_new_tone.grab_focus()
         self.g_repeat.set_sensitive(gtk.FALSE)
         self.g_give_up.set_sensitive(gtk.FALSE)
         self.m_t.q_status = const.QSTATUS_NO
     def on_end_practise(self):
+        self.m_t.end_practise()
         if configureoutput.WITH_GNOME:
             self.g_repeat.set_sensitive(gtk.FALSE)
