@@ -1,5 +1,5 @@
 # GNU Solfege - eartraining for GNOME
-# Copyright (C) 2000, 2001, 2002  Tom Cato Amundsen
+# Copyright (C) 2000, 2001, 2002, 2003  Tom Cato Amundsen
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -15,11 +15,9 @@
 # along with this program; if not, write to the Free Software
 # Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 
-from i18n import _
 import re, string, types, os
 import const
 import random
-import gettext
 import mpd, cfg, utils
 from soundcard.rat import Rat
 
@@ -27,17 +25,18 @@ import dataparser
 
 
 predef = {'dictation': 'dictation',
-                      'progression': 'progression',
-                      'harmony': 'harmony',
-                      'sing-chord': 'sing-chord',
-                      'chord': 'chord',
-                      'id-by-name': 'id-by-name',
-                      'satb': 'satb',
-                      'horiz': 'horiz',
-                      'vertic': 'vertic',
-                      'yes': 1,
-                      'no': 0,
-                      'tempo': (60, 4)}
+          'progression': 'progression',
+          'harmony': 'harmony',
+          'sing-chord': 'sing-chord',
+          'chord-voicing': 'chord-voicing',
+          'chord': 'chord',
+          'id-by-name': 'id-by-name',
+          'satb': 'satb',
+          'horiz': 'horiz',
+          'vertic': 'vertic',
+          'yes': 1,
+          'no': 0,
+          'tempo': (60, 4)}
 
 class _Header:
     def __init__(self, headerdict):
@@ -53,11 +52,13 @@ class _Header:
 
 
 class LessonfileCommon:
+    def __init__(self):
+        self.m_prev_question = None
     def _parse_file(self, filename):
         parser = dataparser.Dataparser(predef, ('tempo',))
         parser.parse_file(filename)
         self._idx = 0
-        self.m_transpose = mpd.MusicalPitch("c'")
+        self.m_transpose = mpd.MusicalPitch.new_from_notename("c'")
         H = filter(lambda e: e['blocktype'] == 'header', parser.m_blocks)
         self.header = _Header(H[0])
         self.m_questions = filter(lambda e: e['blocktype'] == 'question',
@@ -89,7 +90,6 @@ class LessonfileCommon:
         Select a question by random. Set the self.m_transpose variable if
         the lessonfile say the question should be transposed.
         """
-        old_music = self.get_music()
         while 1:
             self._idx = random.randint(0, len(self.m_questions) - 1)
             if self.header.random_transpose == 1:
@@ -98,10 +98,11 @@ class LessonfileCommon:
                 self.m_transpose = mpd.MusicalPitch().randomize(
                   mpd.transpose_notename("c'", self.header.random_transpose[0]),
                   mpd.transpose_notename("c'", self.header.random_transpose[1]))
-            if old_music == self.get_music() \
+            if self.m_prev_question == self.get_music() \
                 and (len(self.m_questions) > 1 or self.header.random_transpose):
                 continue
             break
+        self.m_prev_question = self.get_music()
     def get_tempo(self):
         return self.m_questions[self._idx]['tempo']
     def get_name(self):
@@ -115,18 +116,19 @@ class LessonfileCommon:
         """
         if self.header.musicformat == 'chord':
             return r"\staff\transpose %s{< %s > }" \
-              % (self.m_transpose.str(), self.m_questions[self._idx]['music'])
+              % (self.m_transpose.get_octave_notename(), self.m_questions[self._idx]['music'])
         s = self.m_questions[self._idx]['music']
         if self.header.random_transpose:
             s = string.replace(s, r'\staff',
-               r'\staff\transpose %s' % self.m_transpose.str())
+               r'\staff\transpose %s' % self.m_transpose.get_octave_notename())
             s = string.replace(s, r'\addvoice',
-               r'\addvoice\transpose %s' % self.m_transpose.str())
+               r'\addvoice\transpose %s' % self.m_transpose.get_octave_notename())
         return s
 
 
 class DictationLessonfile(LessonfileCommon):
     def __init__(self, filename):
+        LessonfileCommon.__init__(self)
         self._parse_file(filename)
     def get_breakpoints(self):
         r = []
@@ -152,6 +154,7 @@ class DictationLessonfile(LessonfileCommon):
 
 class SingChordLessonfile(LessonfileCommon):
     def __init__(self, filename):
+        LessonfileCommon.__init__(self)
         self._parse_file(filename)
     def get_satb_vector(self):
         """
@@ -160,14 +163,15 @@ class SingChordLessonfile(LessonfileCommon):
         if self.header.random_transpose:
             v = []
             for x in string.split(self.m_questions[self._idx]['music'], '|'):
-                v.append((mpd.MusicalPitch(x).transpose_by_musicalpitch(self.m_transpose)).str())
+                v.append((mpd.MusicalPitch.new_from_notename(x).transpose_by_musicalpitch(self.m_transpose)).get_octave_notename())
             return v
         return string.split(self.m_questions[self._idx]['music'], '|')
     def get_music(self):
         v = string.split(self.m_questions[self._idx]['music'], '|')
-        k = "c \major"
         if self.m_questions[self._idx].has_key('key'):
             k = self.m_questions[self._idx]['key']
+        else:
+            k = "c \major"
         music = r"""
                 \staff{ \key %s\stemUp <%s> }
                 \addvoice{ \stemDown <%s> }
@@ -176,53 +180,81 @@ class SingChordLessonfile(LessonfileCommon):
                 """ % (k, v[0], v[1], k, v[2], v[3])
         if self.header.random_transpose:
             music = string.replace(music, r"\staff",
-                      r"\staff\transpose %s" % self.m_transpose.str())
+                      r"\staff\transpose %s" % self.m_transpose.get_octave_notename())
             music = string.replace(music, r"\addvoice",
-                      r"\addvoice\transpose %s" % self.m_transpose.str())
+                      r"\addvoice\transpose %s" % self.m_transpose.get_octave_notename())
         return music
     def select_random_question(self):
-        key = "c \major"
-        if self.m_questions[self._idx].has_key('key'):
-            key = self.m_questions[self._idx]['key']
-        v = string.split(key, " ")
-        k = mpd.MusicalPitch(v[0]).str()
-        n = ['des', 'aes', 'ees', 'bes', 'f', 'c',
-             'g', 'd', 'a', 'e', 'b', 'fis', 'cis', 'gis']
-        if k not in n:
-            raise "bad key signature in sing-chord lessonfile"
-        i = n.index(k) - 5
-        if v[1] == '\minor':
-            i = i - 3
-        d = {
-              0: (1, 0, 0, 0),
-             -1: (1, 3, 0, 0),
-             -2: (-1, 1, 0, 0),
-             -3: ( 1, 2, -1, 0),
-             -4: (-1, 2, 0, 0),
-             -5: ( 1, 1, -1, 0),
-             -6: (-1, 3, 1, 0),
-             -7: ( 1, 0, -1, 0),
-             -8: ( 1, 3, -1, 0),
-             -9: (-1, 1, 1, 0),
-             -10: (1, 2, -2, 0),
-             }
-        for x in range(-10, -0):
-            d[-x] = (-d[x][0], d[x][1], d[x][2], d[x][3])
-
-        old_music = self.get_music()
+        """
+        Select a new question by random.
+        """
         while 1:
             self._idx = random.randint(0, len(self.m_questions) - 1)
             if self.header.random_transpose:
-                self.m_transpose = mpd.MusicalPitch("c'") \
-                    + apply(mpd.Intervall()._set, d[random.randint(-5-i, 5-i)])
-            if old_music == self.get_music() \
+                if self.m_questions[self._idx].has_key('key'):
+                    key = self.m_questions[self._idx]['key']
+                else:
+                    key = "c \major"
+                self.m_transpose = self._find_random_transpose(key)
+            if self.m_prev_question == self.get_music() \
                 and (len(self.m_questions) > 1 or self.header.random_transpose):
                 continue
             break
+        self.m_prev_question = self.get_music()
+    def _find_random_transpose(self, key):
+        """
+        Pick a random interval to transpose the question. While doing this,
+        make sure there are not more than MAX_ACC accidentals.
+        """
+        max_acc = 5
+        tone, minmaj = string.split(key, " ")
+        k = mpd.MusicalPitch.new_from_notename(tone).get_octave_notename()
+        #FIXME this list say what key signatures are allowed in sing-chord
+        # lesson files. Get the correct values and document them.
+        kv = ['des', 'aes', 'ees', 'bes', 'f', 'c',
+              'g', 'd', 'a', 'e', 'b', 'fis', 'cis', 'gis']
+        # na tell the number of accidentals (# is positive, b is negative)
+        # the question has from the lessonfile before anything is transpose.
+        na = kv.index(k) - 5
+        if minmaj == '\\minor':
+            na -= 3
+        # the number of steps down the circle of fifths we can go
+        n_down = -max_acc-na
+        # the number of steps up the circle of fifths we can go
+        n_up = max_acc - na
+        # The keys in the dict say how many steps up or down in the
+        # circle of fifths we go if we transpose.
+        d = {
+              -10: '-M6', #c -> eses
+               -9: '-a2', #c -> beses
+               -8: '-a5', #c -> fes
+               -7: '-au', #c -> ces
+               -6: '-a4', #c -> ges
+               -5: 'm2', # c -> des
+               -4: '-M3',# c -> as
+               -3: 'm3', # c -> es
+               -2: '-M2', # c -> bes
+               -1: '4', # c -> f
+                0: 'u',
+                1: '-4', # c -> g,
+                2: 'M2', # c -> d
+                3: '-m3',# c -> a
+                4: 'M3', # c -> e
+                5: '-m2', # c -> b
+                6: 'a4', # c -> fis
+                7: 'au', #c -> cis
+                8: 'a5', # c -> gis
+                9: 'a2', # c -> dis
+                10: 'M6', # c -> ais
+            }
+        interv = mpd.Interval()
+        interv.set_from_string(d[random.choice(range(n_down, n_up+1))])
+        return mpd.MusicalPitch.new_from_notename("c'") + interv
 
 
 class IdByNameLessonfile(LessonfileCommon):
     def __init__(self, filename):
+        LessonfileCommon.__init__(self)
         self.parse_file(filename)
     def parse_file(self, filename):
         self._parse_file(filename)
@@ -251,6 +283,7 @@ class IdByNameLessonfile(LessonfileCommon):
 
 class ChordLessonfile(LessonfileCommon):
     def __init__(self, filename):
+        LessonfileCommon.__init__(self)
         self.parse_file(filename)
     def parse_file(self, filename):
         self._parse_file(filename)
@@ -278,8 +311,8 @@ class ChordLessonfile(LessonfileCommon):
             return self.m_questions[self._idx]['music']
         v = []
         for n in string.split(self.m_questions[self._idx]['music']):
-            v.append(mpd.MusicalPitch(n).transpose_by_musicalpitch(
-                 self.m_transpose).str())
+            v.append(mpd.MusicalPitch.new_from_notename(n).transpose_by_musicalpitch(
+                 self.m_transpose).get_octave_notename())
         return string.join(v)
     def get_inversion(self):
         """
@@ -318,7 +351,7 @@ class LessonFileManager:
                     p = dataparser.Dataparser(predef)
                     p.parse_file(self.get_complete_filename(collection, filename), 1)
                 except dataparser.DataparserException:
-                    self.m_invalid_files.append(filename)
+                    self.m_invalid_files.append((collection, filename))
                     continue
                 c = p.m_blocks[0]['content']
                 if type(c) == types.StringType:
@@ -334,17 +367,23 @@ class LessonFileManager:
                 you start Solfege. As you can see, Solfege need more
                 lessonfiles. <a href="lessonfiles.html">Write one!</a>"""
         for collection in self.m_db.keys():
-            self.m_htmldoc = self.m_htmldoc + "<h1>%s</h1>" % collection
+            self.m_htmldoc = self.m_htmldoc + "<h1>collection: %s</h1>" % collection
             for use in self.m_db[collection].keys():
                 self.m_htmldoc = self.m_htmldoc + "<h2>%s</h2>" % use
                 for file in self.m_db[collection][use]:
                     P = {const.USE_CHORD: 'chord',
+                         const.USE_CHORD_VOICING: 'chord-voicing',
                          const.USE_HARMONY: 'harmonic-progression-dictation',
                          const.USE_ID_BY_NAME: 'id-by-name',
                          const.USE_DICTATION: 'dictation',
                          const.USE_SING_CHORD: 'sing-chord'}[use]
                     self.m_htmldoc = self.m_htmldoc + \
                       "<a href=\"solfege:practise/%s/%s/%s\">%s</a> " % (P, collection, file, file)
+        if self.m_invalid_files:
+            self.m_htmldoc += "<h1>Files we failed to parse</h1><ul>"
+            for collection, filename in self.m_invalid_files:
+                self.m_htmldoc += "<li>%(collection)s in %(filename)s</li>" % {'collection': collection, 'filename': filename}
+            self.m_htmldoc += "</ul>"
         self.m_htmldoc = self.m_htmldoc + "</body></html>"
     def get_collections(self):
         """
