@@ -1,5 +1,5 @@
 # GNU Solfege - eartraining for GNOME
-# Copyright (C) 2000, 2001, 2002  Tom Cato Amundsen
+# Copyright (C) 2000, 2001, 2002, 2003  Tom Cato Amundsen
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -15,9 +15,8 @@
 # along with this program; if not, write to the Free Software
 # Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 
-import gtk, gnome.uiconsts
+import gtk, gnome.ui
 import abstract, const, gu, mpd, soundcard
-from i18n import _
 import random, os
 import string
 import cfg
@@ -43,6 +42,7 @@ class Teacher(abstract.Teacher):
                                    teacher is picky (== you have to solve the
                                    question before a new is asked).
                Teacher.OK : if a new question was created.
+               Teacher.ERR_NO_ELEMS : if no elements are set to be practised.
         """
         if self.m_timeout_handle:
             gtk.timeout_remove(self.m_timeout_handle)
@@ -101,24 +101,36 @@ class Teacher(abstract.Teacher):
             self.q_status = const.QSTATUS_WRONG
             self.m_app.play_sad_sound()
 
-def load_xpm(parent, filename):
-    pix, mask = gtk.create_pixmap_from_xpm(parent, None,
-                                           os.path.join("xpm", filename))
-    return gtk.GtkPixmap(pix, mask)
+def create_png_image(fn):
+    """
+    Create an image by loading a png file from graphics dir
+    """
+    im = gtk.Image()
+    im.set_from_file(os.path.join('graphics', fn)+'.png')
+    im.show()
+    return im
 
-def load_rhythm_xpm(parent, idx):
-    return load_xpm(parent, "rhythm-" + reduce(lambda x, y:\
-              x+y, string.split(RHYTHMS[idx])) + ".xpm")
+def create_rhythm_image(rhythm):
+    """
+    rhythm : a string like 'c8 c8' or 'c8 c16 c16'
+    The image returned is shown.
+    """
+    fn = 'rhythm-%s' % ''.join(rhythm.split())
+    im = gtk.Image()
+    im.set_from_file(os.path.join('graphics', fn)+'.png')
+    #im.set_from_pixbuf(im.get_pixbuf().scale_simple(24, 24, gtk.gdk.INTERP_BILINEAR))
+    im.show()
+    return im
 
-class RhythmViewer(gtk.GtkFrame):
+class RhythmViewer(gtk.Frame):
     def __init__(self, parent):
-        gtk.GtkFrame.__init__(self)
+        gtk.Frame.__init__(self)
         self.set_shadow_type(gtk.SHADOW_IN)
         self.g_parent = parent
-        self.g_box = gtk.GtkHBox()
+        self.g_box = gtk.HBox()
         self.g_box.show()
-        self.g_box.set_spacing(gnome.uiconsts.PAD_SMALL)
-        self.g_box.set_border_width(gnome.uiconsts.PAD)
+        self.g_box.set_spacing(gnome.ui.PAD_SMALL)
+        self.g_box.set_border_width(gnome.ui.PAD)
         self.add(self.g_box)
         self.m_data = []
         # the number of rhythm elements the viewer is supposed to show
@@ -128,7 +140,8 @@ class RhythmViewer(gtk.GtkFrame):
     def set_num_beats(self, i):
         self.m_num_beats = i
     def clear(self):
-        self.g_box.foreach(lambda o: o.destroy())
+        for child in self.g_box.get_children():
+            child.destroy()
         self.m_data = []
     def create_holders(self):
         """
@@ -138,7 +151,7 @@ class RhythmViewer(gtk.GtkFrame):
             gtk.timeout_remove(self.__timeout)
         self.clear()
         for x in range(self.m_num_beats):
-            self.add_elem("holder.xpm")
+            self.g_box.pack_start(create_png_image('holder'), gtk.FALSE)
         self.m_data = []
     def clear_wrong_part(self):
         """When the user have answered the question, this method is used
@@ -151,40 +164,38 @@ class RhythmViewer(gtk.GtkFrame):
         for n in range(self.m_num_beats):
             if self.m_data[n] != self.g_parent.m_t.m_question[n]:
                 break
-        for x in range(n, len(self.g_box.children())):
-            self.g_box.children()[n].destroy()
+        for x in range(n, len(self.g_box.get_children())):
+            self.g_box.get_children()[n].destroy()
         self.m_data = self.m_data[:n]
         for x in range(n, self.m_num_beats):
-            self.add_elem("holder.xpm")
+            self.g_box.pack_start(create_png_image('holder'), gtk.FALSE)
     def add_elem(self, fn):
-        pix = load_xpm(self, fn)
-        pix.show()
-        self.g_box.pack_start(pix, gtk.FALSE)
+        #UGH remove/rename/whatever?
+        im = gtk.Image()
+        im.set_from_file('graphics/%s' % fn)
+        im.show()
+        self.g_box.pack_start(im, gtk.FALSE)
     def add_rhythm_element(self, i):
         assert len(self.m_data) <= self.m_num_beats
-        if len(self.g_box.children()) >= self.m_num_beats:
-            self.g_box.children()[self.m_num_beats-1].destroy()
-        pix = load_rhythm_xpm(self, i)
-        pix.show()
-        self.g_box.pack_start(pix, gtk.FALSE)
-        self.g_box.reorder_child(pix, len(self.m_data))
+        if len(self.g_box.get_children()) >= self.m_num_beats:
+            self.g_box.get_children()[self.m_num_beats-1].destroy()
+        im = create_rhythm_image(RHYTHMS[i])
+        self.g_box.pack_start(im, gtk.FALSE)
+        self.g_box.reorder_child(im, len(self.m_data))
         self.m_data.append(i)
     def backspace(self):
         if len(self.m_data) > 0:
             if self.g_face:
                 self.g_face.destroy()
                 self.g_face = None
-            self.g_box.children()[len(self.m_data)-1].destroy()
-            pix = load_xpm(self, "holder.xpm")
-            pix.show()
-            self.g_box.pack_start(pix, gtk.FALSE)
+            self.g_box.get_children()[len(self.m_data)-1].destroy()
+            self.g_box.pack_start(create_png_image('holder'), gtk.FALSE)
             del self.m_data[-1]
     def mark_wrong(self, idx):
-        box = gtk.GtkVBox()
-        r = self.g_box.children()[idx]
-        self.g_box.children()[idx].reparent(box)
-        pix = load_xpm(self, "rhythm-wrong.xpm")
-        box.pack_start(pix)
+        box = gtk.VBox()
+        r = self.g_box.get_children()[idx]
+        self.g_box.get_children()[idx].reparent(box)
+        box.pack_start(create_png_image('rhythm-wrong'), gtk.FALSE, gtk.FALSE)
         self.g_box.pack_start(box, gtk.FALSE)
         self.g_box.reorder_child(box, idx)
         box.show_all()
@@ -192,30 +203,26 @@ class RhythmViewer(gtk.GtkFrame):
         "return the number of rhythm elements currently viewed"
         return len(self.m_data)
     def sad_face(self):
-        pix = load_xpm(self, "sadface-32x32.xpm")
-        pix.show()
-        self.g_face = ev = gtk.GtkEventBox()
-        ev.connect('button_press_event', self.on_sadface_event)
-        ev.show()
-        ev.add(pix)
-        self.g_box.pack_start(ev, gtk.FALSE)
+        self.g_face = gtk.EventBox()
+        self.g_face.connect('button_press_event', self.on_sadface_event)
+        self.g_face.show()
+        self.g_face.add(create_png_image('sadface-32x32'))
+        self.g_box.pack_start(self.g_face, gtk.FALSE)
     def happy_face(self):
-        pix = load_xpm(self, "happyface-32x32.xpm")
-        pix.show()
-        self.g_face = ev = gtk.GtkEventBox()
-        ev.connect('button_press_event', self.on_happyface_event)
-        ev.show()
-        ev.add(pix)
-        self.g_box.pack_start(ev, gtk.FALSE)
+        self.g_face = gtk.EventBox()
+        self.g_face.connect('button_press_event', self.on_happyface_event)
+        self.g_face.show()
+        self.g_face.add(create_png_image("happyface-32x32"))
+        self.g_box.pack_start(self.g_face, gtk.FALSE)
     def on_happyface_event(self, obj, event):
-        if event.type == gtk.GDK.BUTTON_PRESS and event.button == 1:
+        if event.type == gtk.gdk.BUTTON_PRESS and event.button == 1:
             self.g_parent.new_question()
     def on_sadface_event(self, obj, event):
-        if event.type == gtk.GDK.BUTTON_PRESS and event.button == 1:
+        if event.type == gtk.gdk.BUTTON_PRESS and event.button == 1:
             self.clear_wrong_part()
     def flash(self, s):
         self.clear()
-        l = gtk.GtkLabel(s)
+        l = gtk.Label(s)
         l.set_name("Feedback")
         l.set_alignment(0.0, 0.5)
         l.show()
@@ -229,113 +236,108 @@ class RhythmViewer(gtk.GtkFrame):
 class Gui(abstract.Gui):
     def __init__(self, teacher, window):
         abstract.Gui.__init__(self, teacher, window)
-        self.m_key_bindings = {'new_ak': self.new_question,
-                               'repeat_ak': self.m_t.play_question,
-                               'give_up_ak': self.show_answer,
-                               'backspace_ak': self.on_backspace}
+        self.m_key_bindings = {'backspace_ak': self.on_backspace}
         hbox = gu.bHBox(self.practise_box, gtk.FALSE)
         for i in range(0, 8):
-            hbox.pack_start(self.xpmbutton(i), gtk.FALSE)
+            b = self.pngbutton(i)
+            if i == 0:
+                self.g_first_rhythm_button = b
+            hbox.pack_start(b, gtk.FALSE)
         hbox = gu.bHBox(self.practise_box, gtk.FALSE)
         for i in range(8, 16):
-            hbox.pack_start(self.xpmbutton(i), gtk.FALSE)
+            hbox.pack_start(self.pngbutton(i), gtk.FALSE)
         hbox = gu.bHBox(self.practise_box, gtk.FALSE)
         for i in range(16, 22):
-            hbox.pack_start(self.xpmbutton(i), gtk.FALSE)
+            hbox.pack_start(self.pngbutton(i), gtk.FALSE)
         #-------
-        self.practise_box.pack_start(gtk.GtkHBox(), gtk.FALSE, 
-                                     padding=gnome.uiconsts.PAD_SMALL)
+        self.practise_box.pack_start(gtk.HBox(), gtk.FALSE,
+                                     padding=gnome.ui.PAD_SMALL)
         self.g_rhythm_viewer = RhythmViewer(self)
-        self.g_rhythm_viewer.set_usize(-1, 52)
+        self.g_rhythm_viewer.set_size_request(-1, 52)
         self.g_rhythm_viewer.create_holders()
         self.practise_box.pack_start(self.g_rhythm_viewer, gtk.FALSE)
 
         # action area
-        self.g_new = gu.bButton(self.action_area, _("New"), self.new_question)
-        self.g_repeat = gu.bButton(self.action_area, _("Repeat"),
+        self.g_new = gu.bButton(self.action_area, _("_New"), self.new_question)
+        self.g_repeat = gu.bButton(self.action_area, _("_Repeat"),
                           lambda btn, self=self: self.m_t.play_question())
         self.g_repeat.set_sensitive(gtk.FALSE)
-        self.g_show = gu.bButton(self.action_area, _("Give up"), self.show_answer)
+        self.g_show = gu.bButton(self.action_area, _("_Give up"), self.give_up)
         self.g_show.set_sensitive(gtk.FALSE)
 
-        self.g_backspace = gu.bButton(self.action_area, _("Backspace"),
+        self.g_backspace = gu.bButton(self.action_area, _("_Backspace"),
                      self.on_backspace)
         self.g_backspace.set_sensitive(gtk.FALSE)
         self.practise_box.show_all()
         ##############
         # config_box #
         ##############
-        frame = gtk.GtkFrame(_("Rhythms to use in question"))
+        frame = gtk.Frame(_("Rhythms to use in question"))
         self.config_box.pack_start(frame, gtk.FALSE)
-        vbox = gtk.GtkVBox()
-        vbox.set_border_width(gnome.uiconsts.PAD_SMALL)
+        vbox = gtk.VBox()
+        vbox.set_border_width(gnome.ui.PAD_SMALL)
         frame.add(vbox)
         hbox = gu.bHBox(vbox, gtk.FALSE)
         for i in range(0, 8):
-            hbox.pack_start(self.xpmcheckbutton(i), gtk.FALSE)
+            hbox.pack_start(self.pngcheckbutton(i), gtk.FALSE)
         hbox = gu.bHBox(vbox, gtk.FALSE)
         for i in range(8, 16):
-            hbox.pack_start(self.xpmcheckbutton(i), gtk.FALSE)
+            hbox.pack_start(self.pngcheckbutton(i), gtk.FALSE)
         hbox = gu.bHBox(vbox, gtk.FALSE)
         for i in range(16, 22):
-            hbox.pack_start(self.xpmcheckbutton(i), gtk.FALSE)
+            hbox.pack_start(self.pngcheckbutton(i), gtk.FALSE)
         #--------    
-        self.config_box.pack_start(gtk.GtkHBox(), gtk.FALSE,
-                                   padding=gnome.uiconsts.PAD_SMALL)
+        self.config_box.pack_start(gtk.HBox(), gtk.FALSE,
+                                   padding=gnome.ui.PAD_SMALL)
 
-        table = gtk.GtkTable(2, 4, gtk.FALSE)
+        table = gtk.Table(2, 4, gtk.FALSE)
         self.config_box.pack_start(table, gtk.FALSE)
         ###
-        label = gtk.GtkLabel(_("Number of beats in question:"))
+        label = gtk.Label(_("Number of beats in question:"))
         label.set_alignment(1.0, 0.5)
-        table.attach(label, 0, 1, 0, 1, xpadding=gnome.uiconsts.PAD_SMALL,
+        table.attach(label, 0, 1, 0, 1, xpadding=gnome.ui.PAD_SMALL,
                       xoptions=gtk.FILL)
         table.attach(gu.nSpinButton(self.m_exname, "num_beats",
-                     gtk.GtkAdjustment(4, 1, 100, 1, 10)),
+                     gtk.Adjustment(4, 1, 100, 1, 10)),
                      1, 2, 0, 1, xoptions=gtk.FILL)
-        self.g_rhythm_perc_combo = combo = gu.PercussionNameCombo(self.m_exname, "rhythm_perc", "Side Stick")
-        table.attach(combo, 2, 3, 0, 1, xoptions=gtk.FILL)
-        label = gtk.GtkLabel(_("Count in before question:"))
+        menu = gu.PercussionNameOptionMenu(self.m_exname, "rhythm_perc", "Side Stick")
+        table.attach(menu, 2, 3, 0, 1, xoptions=gtk.FILL)
+        label = gtk.Label(_("Count in before question:"))
         label.set_alignment(1.0, 0.5)
-        table.attach(label, 0, 1, 1, 2, xpadding=gnome.uiconsts.PAD_SMALL,
+        table.attach(label, 0, 1, 1, 2, xpadding=gnome.ui.PAD_SMALL,
                      xoptions=gtk.FILL)
         table.attach(gu.nSpinButton(self.m_exname, "count_in",
-                     gtk.GtkAdjustment(2, 0, 10, 1, 10)),
+                     gtk.Adjustment(2, 0, 10, 1, 10)),
                      1, 2, 1, 2, xoptions=gtk.FILL)
-        combo = gu.PercussionNameCombo(self.m_exname, "countin_perc",
-                                       "Claves")
-        table.attach(combo, 2, 3, 1, 2, xoptions=gtk.FILL)
+        menu = gu.PercussionNameOptionMenu(self.m_exname, "countin_perc", "Claves")
+        table.attach(menu, 2, 3, 1, 2, xoptions=gtk.FILL)
         #-----
-        self.config_box.pack_start(gtk.GtkHBox(), gtk.FALSE,
-                                   padding=gnome.uiconsts.PAD_SMALL)
+        self.config_box.pack_start(gtk.HBox(), gtk.FALSE,
+                                   padding=gnome.ui.PAD_SMALL)
         #------
         hbox = gu.bHBox(self.config_box, gtk.FALSE)
-        hbox.set_spacing(gnome.uiconsts.PAD_SMALL)
+        hbox.set_spacing(gnome.ui.PAD_SMALL)
         hbox.pack_start(gu.nCheckButton(self.m_exname,
                  "not_start_with_rest",
                  _("Don't start the question with a rest")), gtk.FALSE)
-        sep = gtk.GtkVSeparator()
+        sep = gtk.VSeparator()
         hbox.pack_start(sep, gtk.FALSE)
-        hbox.pack_start(gtk.GtkLabel(_("Beats per minute:")), gtk.FALSE)
+        hbox.pack_start(gtk.Label(_("Beats per minute:")), gtk.FALSE)
         spin = gu.nSpinButton(self.m_exname, 'bpm',
-                 gtk.GtkAdjustment(60, 20, 240, 1, 10))
+                 gtk.Adjustment(60, 20, 240, 1, 10))
         hbox.pack_start(spin, gtk.FALSE)
         self._add_auto_new_question_gui(self.config_box)
         self.config_box.show_all()
-    def xpmbutton(self, i):
+    def pngbutton(self, i):
         "used by the constructor"
-        pix = load_rhythm_xpm(self, i)
-        pix.show()
-        btn = gtk.GtkButton()
-        btn.add(pix)
+        btn = gtk.Button()
+        btn.add(create_rhythm_image(RHYTHMS[i]))
         btn.show()
         btn.connect('clicked', self.guess_element, i)
         return btn
-    def xpmcheckbutton(self, i):
-        pix = load_rhythm_xpm(self, i)
-        pix.show()
+    def pngcheckbutton(self, i):
         btn = gu.nCheckButton(self.m_exname, "rhythm_element_%i" % i, default_value=1)
-        btn.add(pix)
+        btn.add(create_rhythm_image(RHYTHMS[i]))
         btn.show()
         return btn
     def on_backspace(self, widget=None):
@@ -372,6 +374,7 @@ class Gui(abstract.Gui):
     def new_question(self, widget=None):
         g = self.m_t.new_question()
         if g == Teacher.OK:
+            self.g_first_rhythm_button.grab_focus()
             self.g_rhythm_viewer.set_num_beats(self.get_int('num_beats'))
             self.g_rhythm_viewer.create_holders()
             self.g_show.set_sensitive(gtk.TRUE)
@@ -381,19 +384,20 @@ class Gui(abstract.Gui):
             self.m_t.play_question()
         elif g == Teacher.ERR_PICKY:
             self.g_rhythm_viewer.flash(_("You have to solve this question first."))
-        elif g == Teacher.ERR_NO_ELEMS:
-            self.g_rhythm_viewer.flash(_("You have to configure this exercise properly"))
         else:
-            raise "NEVER REACH THIS"
+            assert g == Teacher.ERR_NO_ELEMS
+            self.g_repeat.set_sensitive(gtk.FALSE)
+            self.g_rhythm_viewer.flash(_("You have to configure this exercise properly"))
     def on_start_practise(self):
         self.g_rhythm_viewer.flash(_("Click 'New' to begin."))
+        self.g_new.grab_focus()
     def on_end_practise(self):
+        self.m_t.end_practise()
         self.g_new.set_sensitive(gtk.TRUE)
         self.g_repeat.set_sensitive(gtk.FALSE)
         self.g_show.set_sensitive(gtk.FALSE)
         self.g_rhythm_viewer.create_holders()
-        self.m_t.end_practise()
-    def show_answer(self, widget=None):
+    def give_up(self, widget=None):
         if self.m_t.q_status == const.QSTATUS_NO:
             return
         self.g_rhythm_viewer.clear()
@@ -401,6 +405,7 @@ class Gui(abstract.Gui):
             self.g_rhythm_viewer.add_rhythm_element(i)
         self.m_t.q_status = const.QSTATUS_SOLVED
         self.g_new.set_sensitive(gtk.TRUE)
+        self.g_new.grab_focus()
         self.g_show.set_sensitive(gtk.FALSE)
         self.g_backspace.set_sensitive(gtk.FALSE)
 
