@@ -1,12 +1,19 @@
-#!/usr/bin/env python
+#!/usr/bin/python2.2
 
+import pygtk
+pygtk.require("2.0")
 import profile, os
 import types, sys
+if len(sys.argv) < 2:
+    print "\nUsage: ./test/lessonparser-test.py lessonfile-to-test\n"
+    sys.exit()
 
-sys.path.insert(0, "gnomeemu")
-sys.path.append(".")
-sys.path.append("/usr/lib/python1.5/site-packages")
+if os.path.isdir("soundcard"):
+    sys.path.append(".")
+else:
+    sys.path.append("..")
 
+import src.i18n
 import src.lessonfile
 
 def par():
